@@ -8,16 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CheetahCheat.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: CheetahCheat.cxx 2199 2011-01-01 16:04:32Z stephena $
 //============================================================================
 
 #include "Console.hxx"
+#include "Cart.hxx"
+#include "OSystem.hxx"
 #include "CheetahCheat.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
