@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SoundNull.cxx 2029 2010-04-29 12:46:07Z stephena $
+// $Id: SoundNull.cxx 2199 2011-01-01 16:04:32Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -57,7 +57,7 @@ bool SoundNull::load(Serializer& in)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SoundNull::save(Serializer& out)
+bool SoundNull::save(Serializer& out) const
 {
   out.putString("TIASound");
 
