@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Debugger.cxx 2155 2010-10-18 18:39:57Z stephena $
+// $Id: Debugger.cxx 2250 2011-06-09 14:00:30Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -112,15 +112,12 @@ Debugger::Debugger(OSystem* osystem)
   : DialogContainer(osystem),
     myConsole(NULL),
     mySystem(NULL),
+    myDialog(NULL),
     myParser(NULL),
     myCartDebug(NULL),
     myCpuDebug(NULL),
     myRiotDebug(NULL),
     myTiaDebug(NULL),
-    myTiaInfo(NULL),
-    myTiaOutput(NULL),
-    myTiaZoom(NULL),
-    myRom(NULL),
     myBreakPoints(NULL),
     myReadTraps(NULL),
     myWriteTraps(NULL),
@@ -172,19 +169,12 @@ void Debugger::initialize()
 
   const GUI::Rect& r = getDialogBounds();
 
-  delete myBaseDialog;
-  DebuggerDialog *dd = new DebuggerDialog(myOSystem, this,
-                           r.left, r.top, r.width(), r.height());
-  myBaseDialog = dd;
+  delete myBaseDialog;  myBaseDialog = myDialog = NULL;
+  myDialog = new DebuggerDialog(myOSystem, this,
+      r.left, r.top, r.width(), r.height());
+  myBaseDialog = myDialog;
 
-  myPrompt    = dd->prompt();
-  myTiaInfo   = dd->tiaInfo();
-  myTiaOutput = dd->tiaOutput();
-  myTiaZoom   = dd->tiaZoom();
-  myRom       = dd->rom();
-  myMessage   = dd->message();
-
-  myRewindManager = new RewindManager(*myOSystem, *dd->rewindButton());
+  myRewindManager = new RewindManager(*myOSystem, myDialog->rewindButton());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -226,24 +216,41 @@ void Debugger::setConsole(Console* console)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Debugger::start(const string& message, int address)
 {
-  bool result = myOSystem->eventHandler().enterDebugMode();
+  if(myOSystem->eventHandler().enterDebugMode())
+  {
+    // This must be done *after* we enter debug mode,
+    // so the message isn't erased
+    ostringstream buf;
+    buf << message;
+    if(address > -1)
+      buf << valueToString(address);
 
-  // This must be done *after* we enter debug mode,
-  // so the message isn't erased
-  ostringstream buf;
-  buf << message;
-  if(address > -1)
-    buf << valueToString(address);
-
-  myMessage->setEditString(buf.str());
-
-  return result;
+    myDialog->message().setEditString(buf.str());
+    return true;
+  }
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::quit()
+bool Debugger::startWithFatalError(const string& message)
 {
-  myOSystem->eventHandler().leaveDebugMode();
+  if(myOSystem->eventHandler().enterDebugMode())
+  {
+    // This must be done *after* we enter debug mode,
+    // so the dialog is properly shown
+    myDialog->showFatalMessage(message);
+    return true;
+  }
+  return false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Debugger::quit(bool exitrom)
+{
+  if(exitrom)
+    myOSystem->eventHandler().handleEvent(Event::LauncherMode, 1);
+  else
+    myOSystem->eventHandler().leaveDebugMode();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -616,7 +623,7 @@ void Debugger::setStartState()
   saveOldState(false);
 
   // Set the 're-disassemble' flag, but don't do it until the next scheduled time 
-  myRom->invalidate(false);
+  myDialog->rom().invalidate(false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -676,7 +683,7 @@ GUI::Rect Debugger::getStatusBounds() const
   int x1 = tia.right + 1;
   int y1 = 0;
   int x2 = tia.right + 225 + (dlg.width() > 1030 ?
-           (int) (0.3 * (dlg.width() - 1030)) : 0);
+           (int) (0.35 * (dlg.width() - 1030)) : 0);
   int y2 = tia.bottom;
   GUI::Rect r(x1, y1, x2, y2);
 
@@ -818,13 +825,16 @@ void Debugger::getCompletions(const char* in, StringList& list) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::saveROM(const string& filename) const
+string Debugger::saveROM(const string& filename) const
 {
-  ofstream out(filename.c_str(), ios::out | ios::binary);
-  if(out.is_open())
-    return myConsole->cartridge().save(out);
+  string path = AbstractFilesystemNode::getAbsolutePath(filename, "~", "a26");
+  FilesystemNode node(path);
+
+  ofstream out(node.getPath(true).c_str(), ios::out | ios::binary);
+  if(out.is_open() && myConsole->cartridge().save(out))
+    return node.getPath(false);
   else
-    return false;
+    return "";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
