@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DebuggerParser.cxx 2005 2010-04-12 22:26:41Z stephena $
+// $Id: DebuggerParser.cxx 2254 2011-06-11 22:45:50Z stephena $
 //============================================================================
 
 #include <fstream>
@@ -31,9 +31,13 @@
 #include "M6502.hxx"
 #include "Expression.hxx"
 #include "FSNode.hxx"
+#include "PromptWidget.hxx"
 #include "RomWidget.hxx"
+#include "ProgressDialog.hxx"
+#include "PackedBitArray.hxx"
 
 #ifdef CHEATCODE_SUPPORT
+  #include "Cheat.hxx"
   #include "CheatManager.hxx"
 #endif
 
@@ -43,6 +47,8 @@
 #define CALL_METHOD(method) ( (this->*method)() )
 
 
+// TODO - use C++ streams instead of nasty C-strings and pointers
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DebuggerParser::DebuggerParser(Debugger* d)
   : debugger(d)
@@ -107,7 +113,7 @@ string DebuggerParser::run(const string& command)
 
   for(int i = 0; i < kNumCommands; ++i)
   {
-    if(verb == commands[i].cmdString)
+    if(BSPF_equalsIgnoreCase(verb, commands[i].cmdString))
     {
       if(validateArgs(i))
         CALL_METHOD(commands[i].executor);
@@ -129,25 +135,26 @@ string DebuggerParser::exec(const FilesystemNode& file)
   {
     ifstream in(file.getPath().c_str());
     if(!in.is_open())
-      return red("file \'" + file.getRelativePath() + "\' not found.");
+      return red("autoexec file \'" + file.getPath(false) + "\' not found");
 
     ostringstream buf;
     int count = 0;
-    char buffer[256];
+    string command;
     while( !in.eof() )
     {
-      if(!in.getline(buffer, 255))
+      if(!getline(in, command))
         break;
 
+      run(command);
       count++;
     }
     buf << "Executed " << debugger->valueToString(count) << " commands from \""
-        << file.getRelativePath() << "\"";
+        << file.getPath(false) << "\"";
 
     return buf.str();
   }
   else
-    return red("file \'" + file.getRelativePath() + "\' not found.");
+    return red("autoexec file \'" + file.getPath(false) + "\' not found");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -158,9 +165,8 @@ void DebuggerParser::getCompletions(const char* in, StringList& completions) con
   // cerr << "Attempting to complete \"" << in << "\"" << endl;
   for(int i = 0; i < kNumCommands; ++i)
   {
-    const char* l = commands[i].cmdString.c_str();
-    if(BSPF_strncasecmp(l, in, strlen(in)) == 0)
-      completions.push_back(l);
+    if(BSPF_startsWithIgnoreCase(commands[i].cmdString.c_str(), in))
+      completions.push_back(commands[i].cmdString);
   }
 }
 
@@ -428,7 +434,7 @@ bool DebuggerParser::validateArgs(int cmd)
   while(*p != kARG_END_ARGS && *p != kARG_MULTI_BYTE)
   {
     count++;
-    *p++;
+    p++;
   }
 
   // Evil hack: some commands intentionally take multiple arguments
@@ -492,7 +498,7 @@ bool DebuggerParser::validateArgs(int cmd)
         break;
     }
     curCount++;
-    *p++;
+    p++;
 
   } while(*p != kARG_END_ARGS && curCount < argRequiredCount);
 
@@ -625,6 +631,36 @@ void DebuggerParser::executeA()
   debugger->cpuDebug().setA((uInt8)args[0]);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "bank"	 
+void DebuggerParser::executeBank()
+{
+  int banks = debugger->cartDebug().bankCount();
+  if(argCount == 0)
+  {
+    commandResult << debugger->cartDebug().getCartType() << ": ";
+    if(banks < 2)
+      commandResult << red("bankswitching not supported by this cartridge");
+    else
+    {
+      commandResult << "current = " << debugger->valueToString(debugger->cartDebug().getBank())
+                    << " out of " << debugger->valueToString(banks) << " banks";
+    }
+  }
+  else
+  {
+    if(banks == 1)
+      commandResult << red("bankswitching not supported by this cartridge");
+    else if(args[0] >= banks)
+      commandResult << red("invalid bank number (must be 0 to ")
+                    << debugger->valueToString(banks - 1) << ")";
+    else if(debugger->setBank(args[0]))
+      commandResult << "switched bank OK";
+    else
+      commandResult << red("error switching banks (bankswitching may not be supported)");
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "base"
 void DebuggerParser::executeBase()
@@ -666,7 +702,7 @@ void DebuggerParser::executeBreak()
   else
     bp = args[0];
   debugger->toggleBreakPoint(bp);
-  debugger->myRom->invalidate();
+  debugger->rom().invalidate();
 
   if(debugger->breakPoint(bp))
     commandResult << "Set";
@@ -716,7 +752,7 @@ void DebuggerParser::executeCheat()
   for(int arg = 0; arg < argCount; arg++)
   {
     const string& cheat = argStrings[arg];
-    const Cheat* c = debugger->getOSystem()->cheat().add("DBG", cheat);
+    const Cheat* c = debugger->myOSystem->cheat().add("DBG", cheat);
     if(c && c->enabled())
       commandResult << "Cheat code " << cheat << " enabled" << endl;
     else
@@ -736,6 +772,16 @@ void DebuggerParser::executeClearbreaks()
   commandResult << "all breakpoints cleared";
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "clearconfig"
+void DebuggerParser::executeClearconfig()
+{
+  if(argCount == 1)
+    commandResult << debugger->cartDebug().clearConfig(args[0]);
+  else
+    commandResult << debugger->cartDebug().clearConfig();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "cleartraps"
 void DebuggerParser::executeCleartraps()
@@ -756,10 +802,32 @@ void DebuggerParser::executeClearwatches()
 // "cls"
 void DebuggerParser::executeCls()
 {
-  debugger->prompt()->clearScreen();
+  debugger->prompt().clearScreen();
   commandResult << "";
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "code"
+void DebuggerParser::executeCode()
+{
+  if(argCount != 2)
+  {
+    commandResult << red("Specify start and end of range only");
+    return;
+  }
+  else if(args[1] < args[0])
+  {
+    commandResult << red("Start address must be <= end address");
+    return;
+  }
+
+  bool result = debugger->cartDebug().addDirective(
+                  CartDebug::CODE, args[0], args[1]);
+  commandResult << (result ? "added" : "removed") << " CODE directive on range $"
+                << hex << args[0] << " $" << hex << args[1];
+  debugger->rom().invalidate();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "colortest"
 void DebuggerParser::executeColortest()
@@ -779,13 +847,35 @@ void DebuggerParser::executeD()
     debugger->cpuDebug().setD(args[0]);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "data"
+void DebuggerParser::executeData()
+{
+  if(argCount != 2)
+  {
+    commandResult << red("Specify start and end of range only");
+    return;
+  }
+  else if(args[1] < args[0])
+  {
+    commandResult << red("Start address must be <= end address");
+    return;
+  }
+
+  bool result = debugger->cartDebug().addDirective(
+                  CartDebug::DATA, args[0], args[1]);
+  commandResult << (result ? "added" : "removed") << " DATA directive on range $"
+                << hex << args[0] << " $" << hex << args[1];
+  debugger->rom().invalidate();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "define"
 void DebuggerParser::executeDefine()
 {
   // TODO: check if label already defined?
   debugger->cartDebug().addLabel(argStrings[0], args[1]);
-  debugger->myRom->invalidate();
+  debugger->rom().invalidate();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -866,22 +956,10 @@ void DebuggerParser::executeExec()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// "help"
-void DebuggerParser::executeHelp()
+// "exitrom"
+void DebuggerParser::executeExitRom()
 {
-  // Find length of longest command
-  uInt16 clen = 0;
-  for(int i = 0; i < kNumCommands; ++i)
-  {
-    uInt16 len = commands[i].cmdString.length();
-    if(len > clen)  clen = len;
-  }
-
-  for(int i = 0; i < kNumCommands; ++i)
-    commandResult << setw(clen) << right << commands[i].cmdString
-                  << " - " << commands[i].description << endl;
-
-  commandResult << debugger->builtinHelp();
+  debugger->quit(true);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -915,6 +993,70 @@ void DebuggerParser::executeFunction()
     commandResult << red("invalid expression");
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "gfx"
+void DebuggerParser::executeGfx()
+{
+  if(argCount != 2)
+  {
+    commandResult << red("Specify start and end of range only");
+    return;
+  }
+  else if(args[1] < args[0])
+  {
+    commandResult << red("Start address must be <= end address");
+    return;
+  }
+
+  bool result = debugger->cartDebug().addDirective(
+                  CartDebug::GFX, args[0], args[1]);
+  commandResult << (result ? "added" : "removed") << " GFX directive on range $"
+                << hex << args[0] << " $" << hex << args[1];
+  debugger->rom().invalidate();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "help"
+void DebuggerParser::executeHelp()
+{
+  // Find length of longest command
+  uInt16 clen = 0;
+  for(int i = 0; i < kNumCommands; ++i)
+  {
+    uInt16 len = commands[i].cmdString.length();
+    if(len > clen)  clen = len;
+  }
+
+  commandResult << setfill(' ');
+  for(int i = 0; i < kNumCommands; ++i)
+    commandResult << setw(clen) << right << commands[i].cmdString
+                  << " - " << commands[i].description << endl;
+
+  commandResult << debugger->builtinHelp();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "jump"
+void DebuggerParser::executeJump()
+{
+  int line = -1;
+  int address = args[0];
+
+  // The specific address we want may not exist (it may be part of a data section)
+  // If so, scroll backward a little until we find it
+  while(((line = debugger->cartDebug().addressToLine(address)) == -1) &&
+        ((address & 0xFFF) >= 0))
+    address--;
+
+  if(line >= 0 && address >= 0)
+  {
+    debugger->rom().scrollTo(line);
+    commandResult << "disassembly scrolled to address $" << HEX4 << address;
+  }
+  else
+    commandResult << "address $" << HEX4 << args[0] << " doesn't exist";
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "listbreaks"
 void DebuggerParser::executeListbreaks()
@@ -955,6 +1097,16 @@ void DebuggerParser::executeListbreaks()
     commandResult << "no breakpoints set";
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "listconfig"
+void DebuggerParser::executeListconfig()
+{
+  if(argCount == 1)
+    commandResult << debugger->cartDebug().listConfig(args[0]);
+  else
+    commandResult << debugger->cartDebug().listConfig();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "listfunctions"
 void DebuggerParser::executeListfunctions()
@@ -990,15 +1142,24 @@ void DebuggerParser::executeListtraps()
     commandResult << "no traps set";
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "loadconfig"
+void DebuggerParser::executeLoadconfig()
+{
+  if(argCount == 1)
+    commandResult << debugger->cartDebug().loadConfigFile(argStrings[0]);
+  else
+    commandResult << debugger->cartDebug().loadConfigFile();
+
+  debugger->rom().invalidate();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "loadstate"
 void DebuggerParser::executeLoadstate()
 {
   if(args[0] >= 0 && args[0] <= 9)
-  {
     debugger->loadState(args[0]);
-    commandResult << "state loaded";
-  }
   else
     commandResult << red("invalid slot (must be 0-9)");
 }
@@ -1008,7 +1169,7 @@ void DebuggerParser::executeLoadstate()
 void DebuggerParser::executeLoadsym()
 {
   commandResult << debugger->cartDebug().loadSymbolFile(argStrings[0]);
-  debugger->myRom->invalidate();
+  debugger->rom().invalidate();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1028,6 +1189,28 @@ void DebuggerParser::executePc()
   debugger->cpuDebug().setPC(args[0]);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "pgfx"
+void DebuggerParser::executePGfx()
+{
+  if(argCount != 2)
+  {
+    commandResult << red("Specify start and end of range only");
+    return;
+  }
+  else if(args[1] < args[0])
+  {
+    commandResult << red("Start address must be <= end address");
+    return;
+  }
+
+  bool result = debugger->cartDebug().addDirective(
+                  CartDebug::PGFX, args[0], args[1]);
+  commandResult << (result ? "added" : "removed") << " PGFX directive on range $"
+                << hex << args[0] << " $" << hex << args[1];
+  debugger->rom().invalidate();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "print"
 void DebuggerParser::executePrint()
@@ -1050,7 +1233,7 @@ void DebuggerParser::executeRam()
 void DebuggerParser::executeReset()
 {
   debugger->reset();
-  debugger->myRom->invalidate();
+  debugger->rom().invalidate();
   commandResult << "reset CPU";
 }
 
@@ -1060,7 +1243,7 @@ void DebuggerParser::executeRewind()
 {
   if(debugger->rewindState())
   {
-    debugger->myRom->invalidate();
+    debugger->rom().invalidate();
     commandResult << "rewind by one level";
   }
   else
@@ -1093,18 +1276,40 @@ void DebuggerParser::executeRom()
   // The RomWidget is a special case, since we don't want to re-disassemble
   // any more than necessary.  So we only do it by calling the following
   // method ...
-  debugger->myRom->invalidate();
+  debugger->rom().invalidate();
 
   commandResult << "changed " << debugger->valueToString( args.size() - 1 )
                 << " location(s)";
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "row"
+void DebuggerParser::executeRow()
+{
+  if(argCount != 2)
+  {
+    commandResult << red("Specify start and end of range only");
+    return;
+  }
+  else if(args[1] < args[0])
+  {
+    commandResult << red("Start address must be <= end address");
+    return;
+  }
+
+  bool result = debugger->cartDebug().addDirective(
+                  CartDebug::ROW, args[0], args[1]);
+  commandResult << (result ? "added" : "removed") << " ROW directive on range $"
+                << hex << args[0] << " $" << hex << args[1];
+  debugger->rom().invalidate();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "run"
 void DebuggerParser::executeRun()
 {
   debugger->saveOldState();
-  debugger->quit();
+  debugger->quit(false);
   commandResult << "_EXIT_DEBUGGER";  // See PromptWidget for more info
 }
 
@@ -1113,9 +1318,18 @@ void DebuggerParser::executeRun()
 void DebuggerParser::executeRunTo()
 {
   const CartDebug& cartdbg = debugger->cartDebug();
-  const CartDebug::DisassemblyList& list = cartdbg.disassemblyList();
+  const CartDebug::DisassemblyList& list = cartdbg.disassembly().list;
+
+  uInt32 count = 0, max_iterations = list.size();
+
+  // Create a progress dialog box to show the progress searching through the
+  // disassembly, since this may be a time-consuming operation
+  ostringstream buf;
+  buf << "RunTo searching through " << max_iterations << " disassembled instructions";
+  ProgressDialog progress(debugger->myBaseDialog,
+      debugger->myOSystem->consoleFont(), buf.str());
+  progress.setRange(0, max_iterations, 5);
 
-  uInt32 count = 0;
   bool done = false;
   do {
     debugger->step();
@@ -1125,10 +1339,13 @@ void DebuggerParser::executeRunTo()
     if(pcline >= 0)
     {
       const string& next = list[pcline].disasm;
-      done = (next.find(argStrings[0]) != string::npos);
+      done = (BSPF_findIgnoreCase(next, argStrings[0]) != string::npos);
     }
-    ++count;
-  } while(!done && count < list.size());
+    // Update the progress bar
+    progress.setProgress(count);
+  } while(!done && ++count < max_iterations);
+
+  progress.close();
 
   if(done)
     commandResult
@@ -1147,7 +1364,7 @@ void DebuggerParser::executeRunTo()
 void DebuggerParser::executeRunToPc()
 {
   const CartDebug& cartdbg = debugger->cartDebug();
-  const CartDebug::DisassemblyList& list = cartdbg.disassemblyList();
+  const CartDebug::DisassemblyList& list = cartdbg.disassembly().list;
 
   uInt32 count = 0;
   bool done = false;
@@ -1189,12 +1406,23 @@ void DebuggerParser::executeSave()
     commandResult << red("I/O error");
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "saveconfig"
+void DebuggerParser::executeSaveconfig()
+{
+  if(argCount == 1)
+    commandResult << debugger->cartDebug().saveConfigFile(argStrings[0]);
+  else
+    commandResult << debugger->cartDebug().saveConfigFile();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "saverom"
 void DebuggerParser::executeSaverom()
 {
-  if(debugger->saveROM(argStrings[0]))
-    commandResult << "saved ROM as " << argStrings[0];
+  const string& result = debugger->saveROM(argStrings[0]);
+  if(result != "")
+    commandResult << "saved ROM as " << result;
   else
     commandResult << red("failed to save ROM");
 }
@@ -1203,7 +1431,7 @@ void DebuggerParser::executeSaverom()
 // "saveses"
 void DebuggerParser::executeSaveses()
 {
-  if(debugger->prompt()->saveBuffer(argStrings[0]))
+  if(debugger->prompt().saveBuffer(argStrings[0]))
     commandResult << "saved session to file " << argStrings[0];
   else
     commandResult << red("I/O error");
@@ -1214,10 +1442,7 @@ void DebuggerParser::executeSaveses()
 void DebuggerParser::executeSavestate()
 {
   if(args[0] >= 0 && args[0] <= 9)
-  {
     debugger->saveState(args[0]);
-    commandResult << "state saved";
-  }
   else
     commandResult << red("invalid slot (must be 0-9)");
 }
@@ -1233,6 +1458,31 @@ void DebuggerParser::executeScanline()
   if(count != 1) commandResult << "s";
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "skip"
+void DebuggerParser::executeSkip()
+{
+  commandResult << red("Not yet implemented");
+/*
+  if(argCount != 2)
+  {
+    commandResult << red("Specify start and end of range only");
+    return;
+  }
+  else if(args[1] < args[0])
+  {
+    commandResult << red("Start address must be <= end address");
+    return;
+  }
+
+  bool result = debugger->cartDebug().addDirective(
+                  CartDebug::SKIP, args[0], args[1]);
+  commandResult << (result ? "added" : "removed") << " SKIP directive on range $"
+                << hex << args[0] << " $" << hex << args[1];
+  debugger->myRom->invalidate();
+*/
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "step"
 void DebuggerParser::executeStep()
@@ -1260,25 +1510,62 @@ void DebuggerParser::executeTrace()
 // "trap"
 void DebuggerParser::executeTrap()
 {
-  debugger->toggleReadTrap(args[0]);
-  debugger->toggleWriteTrap(args[0]);
-  commandResult << trapStatus(args[0]);
+  uInt32 beg = args[0];
+  uInt32 end = argCount >= 2 ? args[1] : beg;
+  if(beg > end)  BSPF_swap(beg, end);
+
+  for(uInt32 i = beg; i <= end; ++i)
+  {
+    debugger->toggleReadTrap(i);
+    debugger->toggleWriteTrap(i);
+    commandResult << trapStatus(i) << endl;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "trapread"
 void DebuggerParser::executeTrapread()
 {
-  debugger->toggleReadTrap(args[0]);
-  commandResult << trapStatus(args[0]);
+  uInt32 beg = args[0];
+  uInt32 end = argCount >= 2 ? args[1] : beg;
+  if(beg > end)  BSPF_swap(beg, end);
+
+  for(uInt32 i = beg; i <= end; ++i)
+  {
+    debugger->toggleReadTrap(i);
+    commandResult << trapStatus(i) << endl;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "trapwrite"
 void DebuggerParser::executeTrapwrite()
 {
-  debugger->toggleWriteTrap(args[0]);
-  commandResult << trapStatus(args[0]);
+  uInt32 beg = args[0];
+  uInt32 end = argCount >= 2 ? args[1] : beg;
+  if(beg > end)  BSPF_swap(beg, end);
+
+  for(uInt32 i = beg; i <= end; ++i)
+  {
+    debugger->toggleWriteTrap(i);
+    commandResult << trapStatus(i) << endl;
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// "type"
+void DebuggerParser::executeType()
+{
+  uInt32 beg = args[0];
+  uInt32 end = argCount >= 2 ? args[1] : beg;
+  if(beg > end)  BSPF_swap(beg, end);
+
+  for(uInt32 i = beg; i <= end; ++i)
+  {
+    commandResult << HEX4 << i << ": ";
+    debugger->cartDebug().addressTypeAsString(commandResult, i);
+    commandResult << endl;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1287,7 +1574,7 @@ void DebuggerParser::executeUndef()
 {
   if(debugger->cartDebug().removeLabel(argStrings[0]))
   {
-    debugger->myRom->invalidate();
+    debugger->rom().invalidate();
     commandResult << argStrings[0] + " now undefined";
   }
   else
@@ -1349,6 +1636,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeA
   },
 
+  {
+    "bank",
+    "Show # of banks, or switch to bank xx",
+    false,
+    true,
+    { kARG_WORD, kARG_END_ARGS },
+    &DebuggerParser::executeBank
+  },
+
   {
     "base",
     "Set default base (hex, dec, or bin)",
@@ -1403,6 +1699,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeClearbreaks
   },
 
+  {
+    "clearconfig",
+    "Clear Distella config directives [bank xx]",
+    false,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executeClearconfig
+  },
+
   {
     "cleartraps",
     "Clear all traps",
@@ -1430,6 +1735,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeCls
   },
 
+  {
+    "code",
+    "Mark 'CODE' range in disassembly",
+    true,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executeCode
+  },
+
   {
     "colortest",
     "Show value xx as TIA color",
@@ -1448,6 +1762,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeD
   },
 
+  {
+    "data",
+    "Mark 'DATA' range in disassembly",
+    true,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executeData
+  },
+
   {
     "define",
     "Define label xx for address yy",
@@ -1511,6 +1834,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeExec
   },
 
+  {
+    "exitrom",
+    "Exit emulator, return to ROM launcher",
+    false,
+    false,
+    { kARG_END_ARGS },
+    &DebuggerParser::executeExitRom
+  },
+
   {
     "frame",
     "Advance emulation by xx frames (default=1)",
@@ -1529,6 +1861,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeFunction
   },
 
+  {
+    "gfx",
+    "Mark 'CFX' range in disassembly",
+    true,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executeGfx
+  },
+
   {
     "help",
     "This cruft",
@@ -1538,6 +1879,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeHelp
   },
 
+  {
+    "jump",
+    "Scroll disassembly to address xx",
+    true,
+    false,
+    { kARG_WORD, kARG_END_ARGS },
+    &DebuggerParser::executeJump
+  },
+
   {
     "listbreaks",
     "List breakpoints",
@@ -1547,6 +1897,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeListbreaks
   },
 
+  {
+    "listconfig",
+    "List Distella config directives [bank xx]",
+    false,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executeListconfig
+  },
+
   {
     "listfunctions",
     "List user-defined functions",
@@ -1565,6 +1924,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeListtraps
   },
 
+  {
+    "loadconfig",
+    "Load Distella config file [from file xx]",
+    false,
+    true,
+    { kARG_FILE, kARG_MULTI_BYTE },
+    &DebuggerParser::executeLoadconfig
+  },
+
   {
     "loadstate",
     "Load emulator state xx (0-9)",
@@ -1601,6 +1969,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executePc
   },
 
+  {
+    "pgfx",
+    "Mark 'PGFX' range in disassembly",
+    true,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executePGfx
+  },
+
   {
     "print",
     "Evaluate/print expression xx in hex/dec/binary",
@@ -1655,6 +2032,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeRom
   },
 
+  {
+    "row",
+    "Mark 'ROW' range in disassembly",
+    true,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executeRow
+  },
+
   {
     "run",
     "Exit debugger, return to emulator",
@@ -1700,6 +2086,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeSave
   },
 
+  {
+    "saveconfig",
+    "Save Distella config file [to file xx]",
+    false,
+    false,
+    { kARG_FILE, kARG_MULTI_BYTE },
+    &DebuggerParser::executeSaveconfig
+  },
+
   {
     "saverom",
     "Save (possibly patched) ROM to file xx",
@@ -1736,6 +2131,15 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
     &DebuggerParser::executeScanline
   },
 
+  {
+    "skip",
+    "Mark 'SKIP' range in disassembly",
+    true,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executeSkip
+  },
+
   {
     "step",
     "Single step CPU [with count xx]",
@@ -1765,31 +2169,41 @@ DebuggerParser::Command DebuggerParser::commands[kNumCommands] = {
 
   {
     "trap",
-    "Trap read and write accesses to address xx",
+    "Trap read/write access to address(es) xx [to yy]",
     true,
     false,
-    { kARG_WORD, kARG_END_ARGS },
+    { kARG_WORD, kARG_MULTI_BYTE },
     &DebuggerParser::executeTrap
   },
 
   {
     "trapread",
-    "Trap read accesses to address xx",
+    "Trap read access to address(es) xx [to yy]",
     true,
     false,
-    { kARG_WORD, kARG_END_ARGS },
+    { kARG_WORD, kARG_MULTI_BYTE },
     &DebuggerParser::executeTrapread
   },
 
   {
     "trapwrite",
-    "Trap write accesses to address xx",
+    "Trap write access to address(es) xx [to yy]",
     true,
     false,
-    { kARG_WORD, kARG_END_ARGS },
+    { kARG_WORD, kARG_MULTI_BYTE },
     &DebuggerParser::executeTrapwrite
   },
 
+  {
+    "type",
+    "Show disassembly type for address xx [to yy]",
+    true,
+    false,
+    { kARG_WORD, kARG_MULTI_BYTE },
+    &DebuggerParser::executeType
+  },
+
+
   {
     "undef",
     "Undefine label xx (if defined)",
