@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DiStella.cxx 2002 2010-04-10 21:57:49Z stephena $
+// $Id: DiStella.cxx 2199 2011-01-01 16:04:32Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -22,56 +22,160 @@
 #include "DiStella.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-DiStella::DiStella(CartDebug::DisassemblyList& list, uInt16 start,
+DiStella::DiStella(const CartDebug& dbg, CartDebug::DisassemblyList& list,
+                   CartDebug::BankInfo& info, uInt8* labels, uInt8* directives,
                    bool resolvedata)
-  : myList(list)
+  : myDbg(dbg),
+    myList(list),
+    myLabels(labels),
+    myDirectives(directives)
 {
+  CartDebug::AddressList& addresses = info.addressList;
+  if(addresses.size() == 0)
+    return;
+
   while(!myAddressQueue.empty())
     myAddressQueue.pop();
 
-  myAppData.start  = 0x0000;
-  myAppData.end    = 0x0FFF;
-  myAppData.length = 4096;
+  CartDebug::AddressList::iterator it = addresses.begin();
+  uInt16 start = *it++;
 
-  memset(labels, 0, 0x1000);
+  if(start & 0x1000)
+  {
+    if(info.size == 4096)  // 4K ROM space
+    {
+      /*============================================
+        The offset is the address where the code segment
+        starts.  For a 4K game, it is usually 0xf000.
 
-  /*============================================
-    The offset is the address where the code segment
-    starts.  For a 4K game, it is usually 0xf000,
-    which would then have the code data end at 0xffff,
-    but that is not necessarily the case.  Because the
-    Atari 2600 only has 13 address lines, it's possible
-    that the "code" can be considered to start in a lot
-    of different places.  So, we use the start
-    address as a reference to determine where the
-    offset is, logically-anded to produce an offset
-    that is a multiple of 4K.
+        Example:
+          Start address = $D973, so therefore
+          Offset to code = $D000
+          Code range = $D000-$DFFF
+      =============================================*/
+      myAppData.start  = 0x0000;
+      myAppData.end    = 0x0FFF;
 
-    Example:
-      Start address = $D973, so therefore
-      Offset to code = $D000
-      Code range = $D000-$DFFF
-  =============================================*/
-  myOffset = (start - (start % 0x1000));
+      myOffset = (start - (start % 0x1000));
+    }
+    else  // 2K ROM space
+    {
+      /*============================================
+        The offset is the address where the code segment
+        starts.  For a 2K game, it is usually 0xf800,
+        but can also be 0xf000.
+      =============================================*/
+      myAppData.start  = 0x0000;
+      myAppData.end    = 0x07FF;
 
+      myOffset = (start & 0xF800);
+    }
+  }
+  else  // ZP RAM
+  {
+    // For now, we assume all accesses below $1000 are zero-page 
+    myAppData.start  = 0x0080;
+    myAppData.end    = 0x00FF;
+
+    myOffset = 0;
+  }
+  myAppData.length = info.size;
+
+  info.start  = myAppData.start;
+  info.end    = myAppData.end;
+  info.offset = myOffset;
+
+  memset(myLabels, 0, 0x1000);
+  memset(myDirectives, 0, 0x1000);
   myAddressQueue.push(start);
 
+  // Process any directives first, as they override automatic code determination
+  processDirectives(info.directiveList);
+
   if(resolvedata)
   {
+    // After we've disassembled from all addresses in the address list,
+    // use all access points determined by Stella during emulation
+    int codeAccessPoint = 0;
+
     while(!myAddressQueue.empty())
     {
       myPC = myAddressQueue.front();
       myPCBeg = myPC;
       myAddressQueue.pop();
       disasm(myPC, 1);
-      for (uInt32 k = myPCBeg; k <= myPCEnd; k++)
-        mark(k, REACHABLE);
+      if(myPCBeg <= myPCEnd)
+      {
+        // Tentatively mark all addresses in the range as CODE
+        // Note that this is a 'best-effort' approach, since
+        // Distella will normally keep going until the end of the
+        // range or branch is encountered
+        // However, addresses *specifically* marked as DATA/GFX/PGFX
+        // in the emulation core indicate that the CODE range has finished
+        // Therefore, we stop at the first such address encountered
+        for (uInt32 k = myPCBeg; k <= myPCEnd; k++)
+        {
+          if(Debugger::debugger().getAccessFlags(k) &
+             (CartDebug::DATA|CartDebug::GFX|CartDebug::PGFX))
+          {
+            myPCEnd = k - 1;
+            break;
+          }
+          mark(k, CartDebug::CODE);
+        }
+      }
+
+      // When we get to this point, all addresses have been processed
+      // starting from the initial one in the address list
+      // If so, process the next one in the list that hasn't already
+      // been marked as CODE
+      // If it *has* been marked, it can be removed from consideration
+      // in all subsequent passes
+      //
+      // Once the address list has been exhausted, we process all addresses
+      // determined during emulation to represent code, which *haven't* already
+      // been considered
+      //
+      // Note that we can't simply add all addresses right away, since
+      // the processing of a single address can cause others to be added in
+      // the ::disasm method
+      // All of these have to be exhausted before considering a new address
+      if(myAddressQueue.empty())
+      {
+        while(it != addresses.end())
+        {
+          uInt16 addr = *it;
+          if(!check_bit(addr-myOffset, CartDebug::CODE))
+          {
+            myAddressQueue.push(addr);
+            ++it;
+            break;
+          }
+          else   // remove this address, it is redundant
+            it = addresses.erase(it);
+        }
+
+        // Stella itself can provide hints on whether an address has ever
+        // been referenced as CODE
+        while(it == addresses.end() && codeAccessPoint <= myAppData.end)
+        {
+          if((Debugger::debugger().getAccessFlags(codeAccessPoint+myOffset) & CartDebug::CODE)
+             && !(myLabels[codeAccessPoint & myAppData.end] & CartDebug::CODE))
+          {
+            myAddressQueue.push(codeAccessPoint+myOffset);
+            ++codeAccessPoint;
+            break;
+          }
+          ++codeAccessPoint;
+        }
+      }
     }
-    
+
     for (int k = 0; k <= myAppData.end; k++)
     {
-      if (!check_bit(labels[k], REACHABLE))
-        mark(k+myOffset, DATA);
+      if (!check_bit(k, CartDebug::SKIP|CartDebug::CODE|CartDebug::GFX|
+                        CartDebug::PGFX|CartDebug::DATA))
+        mark(k+myOffset, CartDebug::ROW);
     }
   }
 
@@ -90,12 +194,14 @@ DiStella::~DiStella()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DiStella::disasm(uInt32 distart, int pass)
 {
-#define HEX4 uppercase << hex << setw(4) << setfill('0')
-#define HEX2 uppercase << hex << setw(2) << setfill('0')
+#define USER_OR_AUTO_LABEL(pre, address, post)        \
+  const string& l = myDbg.getLabel(address, true);    \
+  if(l != EmptyString)  nextline << pre << l << post; \
+  else                  nextline << pre << "L" << HEX4 << address << post;
 
-  uInt8 op, d1, opsrc;
+  uInt8 op, d1;
   uInt16 ad;
-  short amode;
+  AddressingMode addr_mode;
   int bytes=0, labfound=0, addbranch=0;
   stringstream nextline, nextlinebytes;
   myDisasmBuf.str("");
@@ -104,29 +210,56 @@ void DiStella::disasm(uInt32 distart, int pass)
   myPC = distart - myOffset;
   while(myPC <= myAppData.end)
   {
-    if(check_bit(labels[myPC], GFX))
-      /* && !check_bit(labels[myPC], REACHABLE))*/
+    if(check_bit(myPC, CartDebug::GFX|CartDebug::PGFX) && !check_bit(myPC, CartDebug::CODE))
     {
       if (pass == 2)
-        mark(myPC+myOffset,VALID_ENTRY);
+        mark(myPC+myOffset, CartDebug::VALID_ENTRY);
       else if (pass == 3)
       {
-        if (check_bit(labels[myPC],REFERENCED))
-          myDisasmBuf << HEX4 << myPC+myOffset << "'L'" << HEX4 << myPC+myOffset << "'";
+        if (check_bit(myPC, CartDebug::REFERENCED))
+          myDisasmBuf << HEX4 << myPC+myOffset << "'L" << HEX4 << myPC+myOffset << "'";
         else
           myDisasmBuf << HEX4 << myPC+myOffset << "'     '";
 
-        myDisasmBuf << ".byte $" << HEX2 << (int)Debugger::debugger().peek(myPC+myOffset) << " ; ";
-        showgfx(Debugger::debugger().peek(myPC+myOffset));
-        myDisasmBuf << " $" << HEX4 << myPC+myOffset;
-        addEntry();
+        bool isPGfx = check_bit(myPC, CartDebug::PGFX);
+        const string& bit_string = isPGfx ? "\x1f" : "\x1e";
+        uInt8 byte = Debugger::debugger().peek(myPC+myOffset);
+        myDisasmBuf << ".byte $" << HEX2 << (int)byte << "  |";
+        for(uInt8 i = 0, c = byte; i < 8; ++i, c <<= 1)
+          myDisasmBuf << ((c > 127) ? bit_string : " ");
+        myDisasmBuf << "|  $" << HEX4 << myPC+myOffset << "'";
+        if(settings.gfx_format == kBASE_2)
+          myDisasmBuf << Debugger::to_bin_8(byte);
+        else
+          myDisasmBuf << HEX2 << (int)byte;
+        addEntry(isPGfx ? CartDebug::PGFX : CartDebug::GFX);
       }
       myPC++;
     }
-    else if (check_bit(labels[myPC], DATA) && !check_bit(labels[myPC], GFX))
-        /* && !check_bit(labels[myPC],REACHABLE)) {  */
+    else if (check_bit(myPC, CartDebug::DATA) &&
+             !check_bit(myPC, CartDebug::CODE|CartDebug::GFX|CartDebug::PGFX))
     {
-      mark(myPC+myOffset, VALID_ENTRY);
+      if (pass == 2)
+        mark(myPC+myOffset, CartDebug::VALID_ENTRY);
+      else if (pass == 3)
+      {
+        if (check_bit(myPC, CartDebug::REFERENCED))
+          myDisasmBuf << HEX4 << myPC+myOffset << "'L" << HEX4 << myPC+myOffset << "'";
+        else
+          myDisasmBuf << HEX4 << myPC+myOffset << "'     '";
+
+        uInt8 byte = Debugger::debugger().peek(myPC+myOffset);
+        myDisasmBuf << ".byte $" << HEX2 << (int)byte << "              $"
+                    << HEX4 << myPC+myOffset << "'"
+                    << HEX2 << (int)byte;
+        addEntry(CartDebug::DATA);
+      }
+      myPC++;
+    }
+    else if (check_bit(myPC, CartDebug::ROW) &&
+             !check_bit(myPC, CartDebug::CODE|CartDebug::DATA|CartDebug::GFX|CartDebug::PGFX))
+    {
+      mark(myPC+myOffset, CartDebug::VALID_ENTRY);
       if (pass == 3)
       {
         bytes = 1;
@@ -135,65 +268,76 @@ void DiStella::disasm(uInt32 distart, int pass)
       }
       myPC++;
 
-      while (check_bit(labels[myPC], DATA) && !check_bit(labels[myPC], REFERENCED)
-             && !check_bit(labels[myPC], GFX) && pass == 3 && myPC <= myAppData.end)
+      while (check_bit(myPC, CartDebug::ROW) &&
+             !check_bit(myPC, CartDebug::CODE|CartDebug::DATA|CartDebug::GFX|CartDebug::PGFX)
+             && pass == 3 && myPC <= myAppData.end)
       {
-        if (pass == 3)
+        bytes++;
+        if (bytes == 9) // TODO - perhaps make this configurable to size of output area
         {
-          bytes++;
-          if (bytes == 17)
-          {
-            addEntry();
-            myDisasmBuf << "    '     '.byte $" << HEX2 << (int)Debugger::debugger().peek(myPC+myOffset);
-            bytes = 1;
-          }
-          else
-            myDisasmBuf << ",$" << HEX2 << (int)Debugger::debugger().peek(myPC+myOffset);
+          addEntry(CartDebug::ROW);
+          myDisasmBuf << "    '     '.byte $" << HEX2 << (int)Debugger::debugger().peek(myPC+myOffset);
+          bytes = 1;
         }
+        else
+          myDisasmBuf << ",$" << HEX2 << (int)Debugger::debugger().peek(myPC+myOffset);
+
         myPC++;
       }
 
       if (pass == 3)
       {
-        addEntry();
+        addEntry(CartDebug::ROW);
         myDisasmBuf << "    '     ' ";
-        addEntry();
+        addEntry(CartDebug::NONE);
       }
     }
-    else
+    else  // The following sections must be SKIP or CODE
     {
+      // Add label (if any)
+      //
       op = Debugger::debugger().peek(myPC+myOffset);
       /* version 2.1 bug fix */
       if (pass == 2)
-        mark(myPC+myOffset, VALID_ENTRY);
+        mark(myPC+myOffset, CartDebug::VALID_ENTRY);
       else if (pass == 3)
       {
-        if (check_bit(labels[myPC], REFERENCED))
+        if (check_bit(myPC, CartDebug::REFERENCED))
           myDisasmBuf << HEX4 << myPC+myOffset << "'L" << HEX4 << myPC+myOffset << "'";
         else
           myDisasmBuf << HEX4 << myPC+myOffset << "'     '";
       }
 
-      amode = ourLookup[op].addr_mode;
+      // Add opcode mneumonic
+      //
+      addr_mode = ourLookup[op].addr_mode;
       myPC++;
 
+#if 0
+      // FIXME - the following condition is never true
       if (ourLookup[op].mnemonic[0] == '.')
       {
-        amode = IMPLIED;
+        addr_mode = IMPLIED;
         if (pass == 3)
           nextline << ".byte $" << HEX2 << (int)op << " ;";
       }
-
+#endif
       if (pass == 1)
       {
-        opsrc = ourLookup[op].source;
         /* M_REL covers BPL, BMI, BVC, BVS, BCC, BCS, BNE, BEQ
            M_ADDR = JMP $NNNN, JSR $NNNN
            M_AIND = JMP Abs, Indirect */
-        if ((opsrc == M_REL) || (opsrc == M_ADDR) || (opsrc == M_AIND))
-          addbranch = 1;
-        else
-          addbranch = 0;
+        switch(ourLookup[op].source)
+        {
+          case M_REL:
+          case M_ADDR:
+          case M_AIND:
+            addbranch = 1;
+            break;
+          default:
+            addbranch = 0;
+            break;
+        }
       }
       else if (pass == 3)
       {
@@ -201,9 +345,11 @@ void DiStella::disasm(uInt32 distart, int pass)
         nextlinebytes << HEX2 << (int)op << " ";
       }
 
+      // Add operand(s) for PC values outside the app data range
+      //
       if (myPC >= myAppData.end)
       {
-        switch(amode)
+        switch(addr_mode)
         {
           case ABSOLUTE:
           case ABSOLUTE_X:
@@ -214,21 +360,26 @@ void DiStella::disasm(uInt32 distart, int pass)
           {
             if (pass == 3)
             {
-              /* Line information is already printed; append .byte since last instruction will
-                 put recompilable object larger that original binary file */
-              myDisasmBuf << ".byte $" << HEX2 << (int)op;
-              addEntry();
+              /* Line information is already printed; append .byte since last
+                 instruction will put recompilable object larger that original
+                 binary file */
+              myDisasmBuf << ".byte $" << HEX2 << (int)op << "              $"
+                          << HEX4 << myPC+myOffset << "'"
+                          << HEX2 << (int)op;
+              addEntry(CartDebug::DATA);
 
               if (myPC == myAppData.end)
               {
-                if (check_bit(labels[myPC],REFERENCED))
+                if (check_bit(myPC, CartDebug::REFERENCED))
                   myDisasmBuf << HEX4 << myPC+myOffset << "'L" << HEX4 << myPC+myOffset << "'";
                 else
                   myDisasmBuf << HEX4 << myPC+myOffset << "'     '";
 
                 op = Debugger::debugger().peek(myPC+myOffset);  myPC++;
-                myDisasmBuf << ".byte $" << HEX2 << (int)op;
-                addEntry();
+                myDisasmBuf << ".byte $" << HEX2 << (int)op << "              $"
+                            << HEX4 << myPC+myOffset << "'"
+                            << HEX2 << (int)op;
+                addEntry(CartDebug::DATA);
               }
             }
             myPCEnd = myAppData.end + myOffset;
@@ -248,7 +399,7 @@ void DiStella::disasm(uInt32 distart, int pass)
                 /* Line information is already printed, but we can remove the
                    Instruction (i.e. BMI) by simply clearing the buffer to print */
                 myDisasmBuf << ".byte $" << HEX2 << (int)op;
-                addEntry();
+                addEntry(CartDebug::ROW);
                 nextline.str("");
                 nextlinebytes.str("");
               }
@@ -260,11 +411,13 @@ void DiStella::disasm(uInt32 distart, int pass)
 
           default:
             break;
-        }
+        }  // end switch(addr_mode)
       }
 
+      // Add operand(s)
+      //
       /* Version 2.1 added the extensions to mnemonics */
-      switch(amode)
+      switch(addr_mode)
       {
     #if 0
         case IMPLIED:
@@ -285,15 +438,22 @@ void DiStella::disasm(uInt32 distart, int pass)
         case ABSOLUTE:
         {
           ad = Debugger::debugger().dpeek(myPC+myOffset);  myPC+=2;
-          labfound = mark(ad, REFERENCED);
+          labfound = mark(ad, CartDebug::REFERENCED);
           if (pass == 1)
           {
-            if ((addbranch) && !check_bit(labels[ad & myAppData.end], REACHABLE))
+            if (addbranch)
             {
-              if (ad > 0xfff)
-                myAddressQueue.push((ad & myAppData.end) + myOffset);
+              if (!check_bit(ad & myAppData.end, CartDebug::CODE))
+              {
+                if (ad > 0xfff)
+                  myAddressQueue.push((ad & myAppData.end) + myOffset);
 
-              mark(ad, REACHABLE);
+                mark(ad, CartDebug::CODE);
+              }
+            }
+            else if(ad > 0xfff)
+            {
+              mark(ad, CartDebug::DATA);
             }
           }
           else if (pass == 3)
@@ -305,7 +465,7 @@ void DiStella::disasm(uInt32 distart, int pass)
 
             if (labfound == 1)
             {
-              nextline << "L" << HEX4 << ad;
+              USER_OR_AUTO_LABEL("", ad, "");
               nextlinebytes << HEX2 << (int)(ad&0xff) << " " << HEX2 << (int)(ad>>8);
             }
             else if (labfound == 3)
@@ -316,7 +476,7 @@ void DiStella::disasm(uInt32 distart, int pass)
             else if (labfound == 4)
             {
               int tmp = (ad & myAppData.end)+myOffset;
-              nextline << "L" << HEX4 << tmp;
+              USER_OR_AUTO_LABEL("", tmp, "");
               nextlinebytes << HEX2 << (int)(tmp&0xff) << " " << HEX2 << (int)(tmp>>8);
             }
             else
@@ -331,7 +491,7 @@ void DiStella::disasm(uInt32 distart, int pass)
         case ZERO_PAGE:
         {
           d1 = Debugger::debugger().peek(myPC+myOffset);  myPC++;
-          labfound = mark(d1, REFERENCED);
+          labfound = mark(d1, CartDebug::REFERENCED);
           if (pass == 3)
           {
             if (labfound == 2)
@@ -359,8 +519,16 @@ void DiStella::disasm(uInt32 distart, int pass)
         case ABSOLUTE_X:
         {
           ad = Debugger::debugger().dpeek(myPC+myOffset);  myPC+=2;
-          labfound = mark(ad, REFERENCED);
-          if (pass == 3)
+          labfound = mark(ad, CartDebug::REFERENCED);
+          if (pass == 2 && !check_bit(ad & myAppData.end, CartDebug::CODE))
+          {
+            // Since we can't know what address is being accessed unless we also
+            // know the current X value, this is marked as ROW instead of DATA
+            // The processing is left here, however, in case future versions of
+            // the code can somehow track access to CPU registers
+            mark(ad, CartDebug::ROW);
+          }
+          else if (pass == 3)
           {
             if (ad < 0x100)
               nextline << ".wx ";
@@ -369,7 +537,7 @@ void DiStella::disasm(uInt32 distart, int pass)
 
             if (labfound == 1)
             {
-              nextline << "L" << HEX4 << ad << ",X";
+              USER_OR_AUTO_LABEL("", ad, ",X");
               nextlinebytes << HEX2 << (int)(ad&0xff) << " " << HEX2 << (int)(ad>>8);
             }
             else if (labfound == 3)
@@ -380,7 +548,7 @@ void DiStella::disasm(uInt32 distart, int pass)
             else if (labfound == 4)
             {
               int tmp = (ad & myAppData.end)+myOffset;
-              nextline << "L" << HEX4 << tmp << ",X";
+              USER_OR_AUTO_LABEL("", tmp, ",X");
               nextlinebytes << HEX2 << (int)(tmp&0xff) << " " << HEX2 << (int)(tmp>>8);
             }
             else
@@ -395,8 +563,16 @@ void DiStella::disasm(uInt32 distart, int pass)
         case ABSOLUTE_Y:
         {
           ad = Debugger::debugger().dpeek(myPC+myOffset);  myPC+=2;
-          labfound = mark(ad, REFERENCED);
-          if (pass == 3)
+          labfound = mark(ad, CartDebug::REFERENCED);
+          if (pass == 2 && !check_bit(ad & myAppData.end, CartDebug::CODE))
+          {
+            // Since we can't know what address is being accessed unless we also
+            // know the current Y value, this is marked as ROW instead of DATA
+            // The processing is left here, however, in case future versions of
+            // the code can somehow track access to CPU registers
+            mark(ad, CartDebug::ROW);
+          }
+          else if (pass == 3)
           {
             if (ad < 0x100)
               nextline << ".wy ";
@@ -405,7 +581,7 @@ void DiStella::disasm(uInt32 distart, int pass)
 
             if (labfound == 1)
             {
-              nextline << "L" << HEX4 << ad << ",Y";
+              USER_OR_AUTO_LABEL("", ad, ",Y");
               nextlinebytes << HEX2 << (int)(ad&0xff) << " " << HEX2 << (int)(ad>>8);
             }
             else if (labfound == 3)
@@ -416,7 +592,7 @@ void DiStella::disasm(uInt32 distart, int pass)
             else if (labfound == 4)
             {
               int tmp = (ad & myAppData.end)+myOffset;
-              nextline << "L" << HEX4 << tmp << ",Y";
+              USER_OR_AUTO_LABEL("", tmp, ",Y");
               nextlinebytes << HEX2 << (int)(tmp&0xff) << " " << HEX2 << (int)(tmp>>8);
             }
             else
@@ -453,7 +629,7 @@ void DiStella::disasm(uInt32 distart, int pass)
         case ZERO_PAGE_X:
         {
           d1 = Debugger::debugger().peek(myPC+myOffset);  myPC++;
-          labfound = mark(d1, REFERENCED);
+          labfound = mark(d1, CartDebug::REFERENCED);
           if (pass == 3)
           {
             if (labfound == 2)
@@ -470,7 +646,7 @@ void DiStella::disasm(uInt32 distart, int pass)
         case ZERO_PAGE_Y:
         {
           d1 = Debugger::debugger().peek(myPC+myOffset);  myPC++;
-          labfound = mark(d1,REFERENCED);
+          labfound = mark(d1, CartDebug::REFERENCED);
           if (pass == 3)
           {
             if (labfound == 2)
@@ -492,23 +668,25 @@ void DiStella::disasm(uInt32 distart, int pass)
           d1 = Debugger::debugger().peek(myPC+myOffset);  myPC++;
           ad = ((myPC + (Int8)d1) & 0xfff) + myOffset;
 
-          labfound = mark(ad+myOffset, REFERENCED);
+          labfound = mark(ad, CartDebug::REFERENCED);
           if (pass == 1)
           {
-            if ((addbranch) && !check_bit(labels[ad-myOffset], REACHABLE))
+            if ((addbranch) && !check_bit(ad-myOffset, CartDebug::CODE))
             {
               myAddressQueue.push(ad);
-              mark(ad, REACHABLE);
+              mark(ad, CartDebug::CODE);
             }
           }
           else if (pass == 3)
           {
             if (labfound == 1)
-              nextline << "    L" << HEX4 << ad;
+            {
+              USER_OR_AUTO_LABEL("    ", ad, "");
+            }
             else
               nextline << "    $" << HEX4 << ad;
 
-            nextlinebytes << HEX2 << (int)(ad&0xff) << " " << HEX2 << (int)(ad>>8);
+            nextlinebytes << HEX2 << (int)d1;
           }
           break;
         }
@@ -516,8 +694,16 @@ void DiStella::disasm(uInt32 distart, int pass)
         case ABS_INDIRECT:
         {
           ad = Debugger::debugger().dpeek(myPC+myOffset);  myPC+=2;
-          labfound = mark(ad, REFERENCED);
-          if (pass == 3)
+          labfound = mark(ad, CartDebug::REFERENCED);
+          if (pass == 2 && !check_bit(ad & myAppData.end, CartDebug::CODE))
+          {
+            // Since we can't know what address is being accessed unless we also
+            // know the current X value, this is marked as ROW instead of DATA
+            // The processing is left here, however, in case future versions of
+            // the code can somehow track access to CPU registers
+            mark(ad, CartDebug::ROW);
+          }
+          else if (pass == 3)
           {
             if (ad < 0x100)
               nextline << ".ind ";
@@ -525,7 +711,9 @@ void DiStella::disasm(uInt32 distart, int pass)
               nextline << "     ";
           }
           if (labfound == 1)
-            nextline << "(L" << HEX4 << ad << ")";
+          {
+            USER_OR_AUTO_LABEL("(", ad, ")");
+          }
           else if (labfound == 3)
             nextline << "(" << CartDebug::ourIOMnemonic[ad-0x280] << ")";
           else
@@ -534,10 +722,14 @@ void DiStella::disasm(uInt32 distart, int pass)
           nextlinebytes << HEX2 << (int)(ad&0xff) << " " << HEX2 << (int)(ad>>8);
           break;
         }
+
+        default:
+          break;
       } // end switch
 
       if (pass == 1)
       {
+        // RTS/JMP/RTI always indicate the end of a block of CODE
         if (!strcmp(ourLookup[op].mnemonic,"RTS") ||
             !strcmp(ourLookup[op].mnemonic,"JMP") ||
             /* !strcmp(ourLookup[op].mnemonic,"BRK") || */
@@ -549,20 +741,15 @@ void DiStella::disasm(uInt32 distart, int pass)
       }
       else if (pass == 3)
       {
-        myDisasmBuf << nextline.str();;
-        uInt32 nextlinelen = nextline.str().length();
-        if (nextlinelen <= 15)
-        {
-          /* Print spaces to align cycle count data */
-          for (uInt32 charcnt=0;charcnt<15-nextlinelen;charcnt++)
-            myDisasmBuf << " ";
-        }
-        myDisasmBuf << ";" << dec << (int)ourLookup[op].cycles << "'" << nextlinebytes.str();
-        addEntry();
+        // A complete line of disassembly (text, cycle count, and bytes)
+        myDisasmBuf << nextline.str() << "'"
+                    << ";" << dec << (int)ourLookup[op].cycles << "'"
+                    << nextlinebytes.str();
+        addEntry(CartDebug::CODE);
         if (op == 0x40 || op == 0x60)
         {
           myDisasmBuf << "    '     ' ";
-          addEntry();
+          addEntry(CartDebug::NONE);
         }
 
         nextline.str("");
@@ -576,7 +763,7 @@ void DiStella::disasm(uInt32 distart, int pass)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int DiStella::mark(uInt32 address, MarkType bit)
+int DiStella::mark(uInt32 address, uInt8 mask, bool directive)
 {
   /*-----------------------------------------------------------------------
     For any given offset and code range...
@@ -589,8 +776,8 @@ int DiStella::mark(uInt32 address, MarkType bit)
     We sweep for hardware/system equates, which are valid addresses,
     outside the scope of the code/data range.  For these, we mark its
     corresponding hardware/system array element, and return "2" or "3"
-    (depending on which system/hardware element was accessed), or "5"
-    for zero-page RAM.  If this was not the case...
+    (depending on which system/hardware element was accessed).
+    If this was not the case...
 
     Next we check if it is a code "mirror".  For the 2600, address ranges
     are limited with 13 bits, so other addresses can exist outside of the
@@ -603,25 +790,23 @@ int DiStella::mark(uInt32 address, MarkType bit)
     ===========================================================
       $00-$3d =     system equates (WSYNC, etc...); mark the array's element
                     with the appropriate bit; return 2.
-      $0080-$00FF = zero-page RAM; mark the array's element
-                    with the appropriate bit; return 5.
       $0280-$0297 = system equates (INPT0, etc...); mark the array's element
                     with the appropriate bit; return 3.
-      $1000-$1FFF = CODE/DATA, mark the code/data array for the mirrored address
+      $1000-$1FFF = mark the code/data array for the mirrored address
                     with the appropriate bit; return 4.
-      $3000-$3FFF = CODE/DATA, mark the code/data array for the mirrored address
+      $3000-$3FFF = mark the code/data array for the mirrored address
                     with the appropriate bit; return 4.
-      $5000-$5FFF = CODE/DATA, mark the code/data array for the mirrored address
+      $5000-$5FFF = mark the code/data array for the mirrored address
                     with the appropriate bit; return 4.
-      $7000-$7FFF = CODE/DATA, mark the code/data array for the mirrored address
+      $7000-$7FFF = mark the code/data array for the mirrored address
                     with the appropriate bit; return 4.
-      $9000-$9FFF = CODE/DATA, mark the code/data array for the mirrored address
+      $9000-$9FFF = mark the code/data array for the mirrored address
                     with the appropriate bit; return 4.
-      $B000-$BFFF = CODE/DATA, mark the code/data array for the mirrored address
+      $B000-$BFFF = mark the code/data array for the mirrored address
                     with the appropriate bit; return 4.
-      $D000-$DFFF = CODE/DATA, mark the code/data array for the mirrored address
+      $D000-$DFFF = mark the code/data array for the mirrored address
                     with the appropriate bit; return 4.
-      $F000-$FFFF = CODE/DATA, mark the code/data array for the address
+      $F000-$FFFF = mark the code/data array for the address
                     with the appropriate bit; return 1.
       Anything else = invalid, return 0.
     ===========================================================
@@ -629,19 +814,14 @@ int DiStella::mark(uInt32 address, MarkType bit)
 
   if (address >= myOffset && address <= myAppData.end + myOffset)
   {
-    labels[address-myOffset] = labels[address-myOffset] | bit;
+    myLabels[address-myOffset] = myLabels[address-myOffset] | mask;
+    if(directive)  myDirectives[address-myOffset] = mask;
     return 1;
   }
   else if (address >= 0 && address <= 0x3f)
   {
     return 2;
   }
-/* This isn't supported by the core code yet, so why waste time checking
-  else if (address >= 0x80 && address <= 0xff)
-  {
-    return 5;
-  }
-*/
   else if (address >= 0x280 && address <= 0x297)
   {
     return 3;
@@ -649,7 +829,8 @@ int DiStella::mark(uInt32 address, MarkType bit)
   else if (address > 0x1000)
   {
     /* 2K & 4K case */
-    labels[address & myAppData.end] = labels[address & myAppData.end] | bit;
+    myLabels[address & myAppData.end] = myLabels[address & myAppData.end] | mask;
+    if(directive)  myDirectives[address & myAppData.end] = mask;
     return 4;
   }
   else
@@ -657,55 +838,157 @@ int DiStella::mark(uInt32 address, MarkType bit)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DiStella::showgfx(uInt8 c)
+bool DiStella::check_bit(uInt16 address, uInt8 mask) const
 {
-  int i;
+  // The REFERENCED and VALID_ENTRY flags are needed for any inspection of
+  // an address
+  // Since they're set only in the labels array (as the lower two bits),
+  // they must be included in the other bitfields
+  uInt8 label     = myLabels[address & myAppData.end],
+        lastbits  = label & 0x03,
+        directive = myDirectives[address & myAppData.end] & 0xFC,
+        debugger  = Debugger::debugger().getAccessFlags(address | myOffset) & 0xFC;
 
-  myDisasmBuf << "|";
-  for(i = 0;i < 8; i++)
-  {
-    if (c > 127)
-      myDisasmBuf << "X";
-    else
-      myDisasmBuf << " ";
-
-    c = c << 1;
-  }
-  myDisasmBuf << "|";
+  // Any address marked by a manual directive always takes priority
+  if(directive)
+    return (directive | lastbits) & mask;
+  // Next, the results from a dynamic/runtime analysis are used
+  else if((debugger | lastbits) & mask)
+    return true;
+  // Otherwise, default to static analysis from Distella
+  else
+    return label & mask;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DiStella::addEntry()
+bool DiStella::check_range(uInt16 beg, uInt16 end) const
+{
+  if(beg > end)
+  {
+    cerr << "Beginning of range greater than end: start = " << hex << beg
+         << ", end = " << hex << end << endl;
+    return false;
+  }
+  else if(beg > myAppData.end + myOffset)
+  {
+    cerr << "Beginning of range out of range: start = " << hex << beg
+         << ", range = " << hex << (myAppData.end + myOffset) << endl;
+    return false;
+  }
+  else if(beg < myOffset)
+  {
+    cerr << "Beginning of range out of range: start = " << hex << beg
+         << ", offset = " << hex << myOffset << endl;
+    return false;
+  }
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DiStella::addEntry(CartDebug::DisasmType type)
 {
-  const string& line = myDisasmBuf.str();
   CartDebug::DisassemblyTag tag;
 
-  if(line[0] == ' ')
+  // Type
+  tag.type = type;
+
+  // Address
+  myDisasmBuf.seekg(0, ios::beg);
+  if(myDisasmBuf.peek() == ' ')
     tag.address = 0;
   else
     myDisasmBuf >> setw(4) >> hex >> tag.address;
 
-  if(line[5] != ' ')
-    tag.label = line.substr(5, 5);
+  // Only include addresses within the requested range
+  if(tag.address < myAppData.start)
+    goto DONE_WITH_ADD;
 
-  switch(line[11])
+  // Label (a user-defined label always overrides any auto-generated one)
+  myDisasmBuf.seekg(5, ios::beg);
+  if(tag.address)
   {
-    case ' ':
+    tag.label = myDbg.getLabel(tag.address, true);
+    tag.hllabel = true;
+    if(tag.label == EmptyString)
+    {
+      if(myDisasmBuf.peek() != ' ')
+        getline(myDisasmBuf, tag.label, '\'');
+      else if(settings.show_addresses && tag.type == CartDebug::CODE)
+      {
+        // Have addresses indented, to differentiate from actual labels
+        char address[8];
+        sprintf(address, " %X", tag.address);
+        tag.label = address;
+        tag.hllabel = false;
+      }
+    }
+  }
+
+  // Disassembly
+  // Up to this point the field sizes are fixed, until we get to
+  // variable length labels, cycle counts, etc
+  myDisasmBuf.seekg(11, ios::beg);
+  switch(tag.type)
+  {
+    case CartDebug::SKIP:  // TODO - handle this
       tag.disasm = " ";
       break;
-    case '.':
-      tag.disasm = line.substr(11);
+    case CartDebug::CODE:
+      getline(myDisasmBuf, tag.disasm, '\'');
+      getline(myDisasmBuf, tag.ccount, '\'');
+      getline(myDisasmBuf, tag.bytes);
+
+      // Make note of when we override CODE sections from the debugger
+      // It could mean that the code hasn't been accessed up to this point,
+      // but it could also indicate that code will *never* be accessed
+      // Since it is impossible to tell the difference, marking the address
+      // in the disassembly at least tells the user about it
+      if(!(Debugger::debugger().getAccessFlags(tag.address) & CartDebug::CODE)
+         && myAppData.length >= 4096)
+        tag.ccount += " *";
       break;
-    default:
-      tag.disasm = line.substr(11, 17);
-      tag.bytes  = line.substr(29);
+    case CartDebug::GFX:
+    case CartDebug::PGFX:
+      getline(myDisasmBuf, tag.disasm, '\'');
+      getline(myDisasmBuf, tag.bytes);
+      break;
+    case CartDebug::DATA:
+      getline(myDisasmBuf, tag.disasm, '\'');
+      getline(myDisasmBuf, tag.bytes);
+      break;
+    case CartDebug::ROW:
+      getline(myDisasmBuf, tag.disasm);
+      break;
+    default:  // should never happen
+      tag.disasm = " ";
       break;
   }
   myList.push_back(tag);
 
+DONE_WITH_ADD:
+  myDisasmBuf.clear();
   myDisasmBuf.str("");
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DiStella::processDirectives(const CartDebug::DirectiveList& directives)
+{
+  for(CartDebug::DirectiveList::const_iterator i = directives.begin();
+      i != directives.end(); ++i)
+  {
+    const CartDebug::DirectiveTag tag = *i;
+    if(check_range(tag.start, tag.end))
+      for(uInt32 k = tag.start; k <= tag.end; ++k)
+        mark(k, tag.type, true);
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+DiStella::Settings DiStella::settings = {
+  kBASE_2,  // gfx_format
+  true      // show_addresses
+};
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const DiStella::Instruction_tag DiStella::ourLookup[256] = {
 /****  Positive  ****/
