@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CpuWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: CpuWidget.cxx 2199 2011-01-01 16:04:32Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -80,6 +80,16 @@ CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
     new DataGridWidget(boss, font, xpos, ypos, 1, 4, 8, 8, kBASE_2);
   myCpuGridBinValue->setEditable(false);
 
+  // Create a label and 1x3 grid showing the source of data for A/X/Y registers
+  xpos += myCpuGridBinValue->getWidth() + 20;
+  myCpuDataSrcGrid = 
+    new DataGridWidget(boss, font, xpos, ypos, 1, 4, 4, 16, kBASE_16);
+  myCpuDataSrcGrid->setEditable(false);
+  new StaticTextWidget(boss, font, xpos-font.getMaxCharWidth(),
+                       ypos+myCpuDataSrcGrid->getHeight() + 4,
+                       font.getStringWidth("Src Addr"), fontHeight, "Src Addr",
+                       kTextAlignLeft);
+
   // Add labels for other CPU registers
   xpos = x;
   string labels[4] = { "SP:", "A:", "X:", "Y:" };
@@ -276,6 +286,23 @@ void CpuWidget::fillGrid()
   myCpuGridDecValue->setList(alist, vlist, changed);
   myCpuGridBinValue->setList(alist, vlist, changed);
 
+  // Update the data sources for the A/X/Y registers
+  alist.clear(); vlist.clear(); changed.clear();
+  alist.push_back(0);
+  alist.push_back(0);
+  alist.push_back(0);
+
+  vlist.push_back(state.srcS);
+  vlist.push_back(state.srcA);
+  vlist.push_back(state.srcX);
+  vlist.push_back(state.srcY);
+
+  changed.push_back(state.srcS  != oldstate.srcS);
+  changed.push_back(state.srcA  != oldstate.srcA);
+  changed.push_back(state.srcX  != oldstate.srcX);
+  changed.push_back(state.srcY  != oldstate.srcY);
+  myCpuDataSrcGrid->setList(alist, vlist, changed);
+
   // Update the PS register booleans
   changed.clear();
   for(unsigned int i = 0; i < state.PSbits.size(); ++i)
