@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// and the Stella Team
 //
-// See the file "license" for information on usage and redistribution of
+// See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaWidget.cxx,v 1.13 2008/05/15 15:07:29 stephena Exp $
+// $Id: TiaWidget.cxx 2199 2011-01-01 16:04:32Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -25,6 +26,7 @@
 #include "FrameBuffer.hxx"
 #include "GuiObject.hxx"
 #include "OSystem.hxx"
+#include "CartDebug.hxx"
 #include "TIADebug.hxx"
 #include "ToggleBitWidget.hxx"
 #include "TogglePixelWidget.hxx"
@@ -43,56 +45,64 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& font,
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight();
-  int xpos = 10, ypos = 25, lwidth = 4 * font.getMaxCharWidth();
+  int xpos = 10, ypos = 10, lwidth = 4 * font.getMaxCharWidth();
   StaticTextWidget* t;
 
   // Create a 16x1 grid holding byte values with labels
-  myRamGrid = new DataGridWidget(boss, font, xpos + lwidth, ypos,
-                                 16, 1, 2, 8, kBASE_16);
-  myRamGrid->setEditable(false);
-  myRamGrid->setTarget(this);
-  myRamGrid->setID(kRamID);
-  addFocusWidget(myRamGrid);
-
-  t = new StaticTextWidget(boss, font, xpos, ypos + 2,
-                           lwidth-2, fontHeight,
-                           "00:", kTextAlignLeft);
-  for(int col = 0; col < 16; ++col)
+  // Only do so if we have the vertical height
+  // TODO - at some point, viewing RAM may be removed, as I don't know
+  //        how useful it really is
+  if(h >= 400)
   {
-    t = new StaticTextWidget(boss, font, xpos + col*myRamGrid->colWidth() + lwidth + 7,
-                             ypos - lineHeight,
-                             fontWidth, fontHeight,
-                             Debugger::to_hex_4(col),
-                             kTextAlignLeft);
-  }
+    ypos += lineHeight;
+    myRamGrid = new DataGridWidget(boss, font, xpos + lwidth, ypos,
+                                   16, 1, 2, 8, kBASE_16);
+    myRamGrid->setEditable(false);
+    myRamGrid->setTarget(this);
+    myRamGrid->setID(kRamID);
+    addFocusWidget(myRamGrid);
+
+    t = new StaticTextWidget(boss, font, xpos, ypos + 2,
+                             lwidth-2, fontHeight,
+                             "00:", kTextAlignLeft);
+    for(int col = 0; col < 16; ++col)
+    {
+      t = new StaticTextWidget(boss, font, xpos + col*myRamGrid->colWidth() + lwidth + 7,
+                               ypos - lineHeight,
+                               fontWidth, fontHeight,
+                               Debugger::to_hex_4(col),
+                               kTextAlignLeft);
+    }
   
-  xpos = 20;  ypos += 2 * lineHeight;
-  t = new StaticTextWidget(boss, font, xpos, ypos, 6*fontWidth, fontHeight,
-                           "Label:", kTextAlignLeft);
-  xpos += 6*fontWidth + 5;
-  myLabel = new EditTextWidget(boss, font, xpos, ypos-2, 15*fontWidth,
-                               lineHeight, "");
-  myLabel->setEditable(false);
+    xpos = 20;  ypos += 2 * lineHeight;
+    t = new StaticTextWidget(boss, font, xpos, ypos, 6*fontWidth, fontHeight,
+                             "Label:", kTextAlignLeft);
+    xpos += 6*fontWidth + 5;
+    myLabel = new EditTextWidget(boss, font, xpos, ypos-2, 15*fontWidth,
+                                 lineHeight, "");
+    myLabel->setEditable(false);
 
-  xpos += 15*fontWidth + 20;
-  t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
-                           "Dec:", kTextAlignLeft);
-  xpos += 4*fontWidth + 5;
-  myDecValue = new EditTextWidget(boss, font, xpos, ypos-2, 4*fontWidth,
-                                  lineHeight, "");
-  myDecValue->setEditable(false);
+    xpos += 15*fontWidth + 20;
+    t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
+                             "Dec:", kTextAlignLeft);
+    xpos += 4*fontWidth + 5;
+    myDecValue = new EditTextWidget(boss, font, xpos, ypos-2, 4*fontWidth,
+                                    lineHeight, "");
+    myDecValue->setEditable(false);
 
-  xpos += 4*fontWidth + 20;
-  t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
-                           "Bin:", kTextAlignLeft);
-  xpos += 4*fontWidth + 5;
-  myBinValue = new EditTextWidget(boss, font, xpos, ypos-2, 9*fontWidth,
-                                  lineHeight, "");
-  myBinValue->setEditable(false);
+    xpos += 4*fontWidth + 20;
+    t = new StaticTextWidget(boss, font, xpos, ypos, 4*fontWidth, fontHeight,
+                             "Bin:", kTextAlignLeft);
+    xpos += 4*fontWidth + 5;
+    myBinValue = new EditTextWidget(boss, font, xpos, ypos-2, 9*fontWidth,
+                                    lineHeight, "");
+    myBinValue->setEditable(false);
+    ypos += lineHeight + 10;
+  }
 
   // Color registers
   const char* regNames[] = { "COLUP0:", "COLUP1:", "COLUPF:", "COLUBK:" };
-  xpos = 10;  ypos += 3*lineHeight;
+  xpos = 10;  ypos += lineHeight + 5;
   for(int row = 0; row < 4; ++row)
   {
     t = new StaticTextWidget(boss, font, xpos, ypos + row*lineHeight + 2,
@@ -228,7 +238,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& font,
   // P0 register info
   ////////////////////////////
   // grP0
-  xpos = 10;  ypos = 13*lineHeight;
+  xpos = 10;  ypos = buttonY + 2*lineHeight;
   t = new StaticTextWidget(boss, font, xpos, ypos+2,
                            7*fontWidth, fontHeight,
                            "P0: GR:", kTextAlignLeft);
@@ -589,7 +599,8 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
   int addr, value;
   string buf;
 
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
+  CartDebug& cart = dbg.cartDebug();
   TIADebug& tia = dbg.tiaDebug();
 
   switch(cmd)
@@ -743,7 +754,7 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
 
           // We're using the read-addresses here
           // Should we also add write-addresses, or remove this entirely?
-          myLabel->setEditString(dbg.equates().getLabel(addr, true));
+          myLabel->setEditString(cart.getLabel(addr, true));
 
           myDecValue->setEditString(dbg.valueToString(value, kBASE_10));
           myBinValue->setEditString(dbg.valueToString(value, kBASE_2));
@@ -872,20 +883,23 @@ void TiaWidget::fillGrid()
   IntArray vlist;
   BoolArray blist, changed, grNew, grOld;
 
-  Debugger& dbg = instance()->debugger();
+  Debugger& dbg = instance().debugger();
   TIADebug& tia = dbg.tiaDebug();
   TiaState& state    = (TiaState&) tia.getState();
   TiaState& oldstate = (TiaState&) tia.getOldState();
 
   // TIA RAM
-  alist.clear();  vlist.clear();  changed.clear();
-  for(unsigned int i = 0; i < 16; i++)
+  if(myRamGrid)
   {
-    alist.push_back(i);
-    vlist.push_back(state.ram[i]);
-    changed.push_back(state.ram[i] != oldstate.ram[i]);
+    alist.clear();  vlist.clear();  changed.clear();
+    for(unsigned int i = 0; i < 16; i++)
+    {
+      alist.push_back(i);
+      vlist.push_back(state.ram[i]);
+      changed.push_back(state.ram[i] != oldstate.ram[i]);
+    }
+    myRamGrid->setList(alist, vlist, changed);
   }
-  myRamGrid->setList(alist, vlist, changed);
 
   // Color registers
   alist.clear();  vlist.clear();  changed.clear();
@@ -1051,22 +1065,22 @@ void TiaWidget::changeColorRegs()
   switch(addr)
   {
     case kCOLUP0Addr:
-      instance()->debugger().tiaDebug().coluP0(value);
+      instance().debugger().tiaDebug().coluP0(value);
       myCOLUP0Color->setColor(value);
       break;
 
     case kCOLUP1Addr:
-      instance()->debugger().tiaDebug().coluP1(value);
+      instance().debugger().tiaDebug().coluP1(value);
       myCOLUP1Color->setColor(value);
       break;
 
     case kCOLUPFAddr:
-      instance()->debugger().tiaDebug().coluPF(value);
+      instance().debugger().tiaDebug().coluPF(value);
       myCOLUPFColor->setColor(value);
       break;
 
     case kCOLUBKAddr:
-      instance()->debugger().tiaDebug().coluBK(value);
+      instance().debugger().tiaDebug().coluBK(value);
       myCOLUBKColor->setColor(value);
       break;
   }
