@@ -8,26 +8,32 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartDPCPlus.cxx 2030 2010-05-02 22:52:59Z stephena $
+// $Id: CartDPCPlus.cxx 2234 2011-05-26 16:14:46Z stephena $
 //============================================================================
 
 #include <cassert>
 #include <cstring>
 
+#ifdef DEBUGGER_SUPPORT
+  #include "Debugger.hxx"
+#endif
 #include "System.hxx"
+#include "Thumbulator.hxx"
 #include "CartDPCPlus.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeDPCPlus::CartridgeDPCPlus(const uInt8* image, uInt32 size)
-  : myFastFetch(false),
+CartridgeDPCPlus::CartridgeDPCPlus(const uInt8* image, uInt32 size,
+                                   const Settings& settings)
+  : Cartridge(settings),
+    myFastFetch(false),
     myLDAimmediate(false),
-    myParameter(0),
+    myParameterPointer(0),
     mySystemCycles(0),
     myFractionalClocks(0.0)
 {
@@ -35,16 +41,19 @@ CartridgeDPCPlus::CartridgeDPCPlus(const uInt8* image, uInt32 size)
   uInt32 minsize = 4096 * 6 + 4096 + 1024 + 255;
   mySize = BSPF_max(minsize, size);
   myImage = new uInt8[mySize];
+  myDPCRAM = new uInt8[8192];
   memcpy(myImage, image, size);
+  createCodeAccessBase(4096 * 6);
 
   // Pointer to the program ROM (24K @ 0 byte offset)
   myProgramImage = myImage;
 
-  // Pointer to the display ROM (4K @ 24K offset)
-  myDisplayImage = myProgramImage + 4096 * 6;
+  // Pointer to the display RAM
+  myDisplayImage = myDPCRAM + 0xC00;
+  memset(myDPCRAM, 0, 8192);
 
   // Pointer to the Frequency ROM (1K @ 28K offset)
-  myFrequencyImage = myDisplayImage + 4096;
+  myFrequencyImage = myProgramImage + 0x7000;
 
   // If the image is larger than 29K, we assume any excess at the
   // beginning is ARM code, and skip over it
@@ -52,13 +61,23 @@ CartridgeDPCPlus::CartridgeDPCPlus(const uInt8* image, uInt32 size)
   {
     int offset = size - 29 * 1024;
     myProgramImage   += offset;
-    myDisplayImage   += offset;
+//    myDisplayImage   += offset;
     myFrequencyImage += offset;
   }
 
+#ifdef THUMB_SUPPORT
+  // Create Thumbulator ARM emulator
+  myThumbEmulator = new Thumbulator((uInt16*)(myProgramImage-0xC00),
+                                    (uInt16*)myDPCRAM);
+#endif
+
+  // Copy DPC display data to Harmony RAM
+  memcpy(myDisplayImage, myProgramImage + 0x6000, 0x1000);
+
   // Initialize the DPC data fetcher registers
   for(uInt16 i = 0; i < 8; ++i)
-    myTops[i] = myBottoms[i] = myCounters[i] = myFlags[i] = myFractionalIncrements[i] = 0;
+    myTops[i] = myBottoms[i] = myCounters[i] = myFractionalIncrements[i] = 
+    myFractionalCounters[i] = 0;
 
   // Set waveforms to first waveform entry
   myMusicWaveforms[0] = myMusicWaveforms[1] = myMusicWaveforms[2] = 0;
@@ -74,6 +93,11 @@ CartridgeDPCPlus::CartridgeDPCPlus(const uInt8* image, uInt32 size)
 CartridgeDPCPlus::~CartridgeDPCPlus()
 {
   delete[] myImage;
+  delete[] myDPCRAM;
+
+#ifdef THUMB_SUPPORT
+  delete myThumbEmulator;
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -107,13 +131,10 @@ void CartridgeDPCPlus::install(System& system)
   // Make sure the system we're being installed in has a page size that'll work
   assert(((0x1080 & mask) == 0) && ((0x1100 & mask) == 0));
 
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
+
   // Map all of the accesses to call peek and poke
-  System::PageAccess access;
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
-  for(uInt32 i = 0x1000; i < 0x2000; i += (1 << shift))
+  for(uInt32 i = 0x1000; i < 0x1080; i += (1 << shift))
     mySystem->setPageAccess(i >> shift, access);
 
   // Install pages for the startup bank
@@ -124,14 +145,17 @@ void CartridgeDPCPlus::install(System& system)
 inline void CartridgeDPCPlus::clockRandomNumberGenerator()
 {
   // Update random number generator (32-bit LFSR)
-  myRandomNumber = ((myRandomNumber & 1) ? 0xa260012b: 0x00) ^ ((myRandomNumber >> 1) & 0x7FFFFFFF);
+  myRandomNumber = ((myRandomNumber & (1<<10)) ? 0x10adab1e: 0x00) ^
+                   ((myRandomNumber >> 11) | (myRandomNumber << 21));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 inline void CartridgeDPCPlus::priorClockRandomNumberGenerator()
 {
   // Update random number generator (32-bit LFSR, reversed)
-  myRandomNumber = ((myRandomNumber & (1<<31)) ? 0x44c00257: 0x00) ^ (myRandomNumber << 1);
+  myRandomNumber = ((myRandomNumber & (1<<31)) ?
+    ((0x10adab1e^myRandomNumber) << 11) | ((0x10adab1e^myRandomNumber) >> 21) :
+    (myRandomNumber << 11) | (myRandomNumber >> 21));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -161,15 +185,42 @@ inline void CartridgeDPCPlus::updateMusicModeDataFetchers()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 inline void CartridgeDPCPlus::callFunction(uInt8 value)
 {
-  //  myParameter
+  // myParameter
+  uInt16 ROMdata = (myParameter[1] << 8) + myParameter[0];
   switch (value)
   {
-    //case 0:     template for future special functions
-      // CallSpecialFunctionO(myParameter);
-      // break;
-    case 255: // Call user writen ARM code (most likely be C compiled for ARM).
-              // ARM code not supported by Stella at this time.
+    case 0: // Parameter Pointer reset
+      myParameterPointer = 0;
       break;
+    case 1: // Copy ROM to fetcher
+      for(int i = 0; i < myParameter[3]; ++i)
+        myDisplayImage[myCounters[myParameter[2] & 0x7]+i] = myProgramImage[ROMdata+i];
+      myParameterPointer = 0;
+      break;
+    case 2: // Copy value to fetcher
+      for(int i = 0; i < myParameter[3]; ++i)
+        myDisplayImage[myCounters[myParameter[2]]+i] = myParameter[0];
+      myParameterPointer = 0;
+      break;
+  #ifdef THUMB_SUPPORT
+    case 254:
+    case 255:
+      // Call user written ARM code (most likely be C compiled for ARM)
+      try {
+        myThumbEmulator->run();
+      }
+      catch(const string& error) {
+        if(!mySystem->autodectMode())
+        {
+      #ifdef DEBUGGER_SUPPORT
+          Debugger::debugger().startWithFatalError(error);
+      #else
+          cout << error << endl;
+      #endif
+        }
+      }
+      break;
+  #endif
     // reserved
   }
 }
@@ -180,6 +231,7 @@ uInt8 CartridgeDPCPlus::peek(uInt16 address)
   address &= 0x0FFF;
 
   uInt8 peekvalue = myProgramImage[(myCurrentBank << 12) + address];
+  uInt8 flag;
 
   // In debugger/bank-locked mode, we ignore all hotspots and in general
   // anything that can change the internal state of the cart
@@ -203,16 +255,9 @@ uInt8 CartridgeDPCPlus::peek(uInt16 address)
     uInt32 index = address & 0x07;
     uInt32 function = (address >> 3) & 0x07;
 
-    // Update flag register for selected data fetcher
-    if((myCounters[index] & 0x00ff) == ((myTops[index]+1) & 0xff))
-    {
-      myFlags[index] = 0xff;
-    }
-    else if((myCounters[index] & 0x00ff) == myBottoms[index])
-    {
-      myFlags[index] = 0x00;
-    }
-
+    // Update flag for selected data fetcher  
+    flag = (((myTops[index]-(myCounters[index] & 0x00ff)) & 0xFF) > ((myTops[index]-myBottoms[index]) & 0xFF)) ? 0xFF : 0;
+    
     switch(function)
     {
       case 0x00:
@@ -274,7 +319,7 @@ uInt8 CartridgeDPCPlus::peek(uInt16 address)
       // DFxDATAW - display data read AND'd w/flag ("windowed")
       case 0x02:
       {
-        result = myDisplayImage[myCounters[index]] & myFlags[index];
+        result = myDisplayImage[myCounters[index]] & flag;
         myCounters[index] = (myCounters[index] + 0x1) & 0x0fff;
         break;
       }
@@ -296,7 +341,7 @@ uInt8 CartridgeDPCPlus::peek(uInt16 address)
           case 0x02:  // DF2FLAG
           case 0x03:  // DF3FLAG
           {
-            result = myFlags[index];
+            result = flag;
             break;
           }
           case 0x04:  // reserved
@@ -394,7 +439,6 @@ bool CartridgeDPCPlus::poke(uInt16 address, uInt8 value)
       // DFxTOP - set top of window (for reads of DFxDATAW)
       case 0x03:
         myTops[index] = value;
-        myFlags[index] = 0x00;
         break;
 
       // DFxBOT - set bottom of window (for reads of DFxDATAW)
@@ -416,7 +460,8 @@ bool CartridgeDPCPlus::poke(uInt16 address, uInt8 value)
             break;
 
           case 0x01:  // PARAMETER - set parameter used by CALLFUNCTION (not all functions use the parameter)
-            myParameter = value;
+            if(myParameterPointer < 8)
+              myParameter[myParameterPointer++] = value;
             break;
 
           case 0x02:  // CALLFUNCTION
@@ -552,14 +597,25 @@ bool CartridgeDPCPlus::poke(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeDPCPlus::bank(uInt16 bank)
+bool CartridgeDPCPlus::bank(uInt16 bank)
 {
-  if(bankLocked()) return;
+  if(bankLocked()) return false;
 
   // Remember what bank we're in
   myCurrentBank = bank;
+  uInt16 offset = myCurrentBank << 12;
+  uInt16 shift = mySystem->pageShift();
 
-  myBankChanged = true;
+  // Setup the page access methods for the current bank
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
+
+  // Map Program ROM image into the system
+  for(uInt32 address = 0x1080; address < 0x2000; address += (1 << shift))
+  {
+    access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x0FFF)];
+    mySystem->setPageAccess(address >> shift, access);
+  }
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -599,13 +655,11 @@ const uInt8* CartridgeDPCPlus::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeDPCPlus::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
     uInt32 i;
 
-    out.putString(cart);
+    out.putString(name());
 
     // Indicates which bank is currently active
     out.putInt(myCurrentBank);
@@ -635,17 +689,14 @@ bool CartridgeDPCPlus::save(Serializer& out) const
     for(i = 0; i < 8; ++i)
       out.putByte((char)myFractionalIncrements[i]);
 
-    // The flag registers for the data fetchers
-    out.putInt(8);
-    for(i = 0; i < 8; ++i)
-      out.putByte((char)myFlags[i]);
-
     // The Fast Fetcher Enabled flag
     out.putBool(myFastFetch);
     out.putBool(myLDAimmediate);
 
     // Control Byte to update
-    out.putByte((char) myParameter);
+    out.putInt(8);
+    for(i = 0; i < 8; ++i)
+      out.putByte((char)myParameter[i]);
 
     // The music counters
     out.putInt(3);
@@ -680,11 +731,9 @@ bool CartridgeDPCPlus::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeDPCPlus::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     uInt32 i, limit;
@@ -717,17 +766,14 @@ bool CartridgeDPCPlus::load(Serializer& in)
     for(i = 0; i < limit; ++i)
       myFractionalIncrements[i] = (uInt8) in.getByte();
 
-    // The flag registers for the data fetchers
-    limit = (uInt32) in.getInt();
-    for(i = 0; i < limit; ++i)
-      myFlags[i] = (uInt8) in.getByte();
-
     // The Fast Fetcher Enabled flag
     myFastFetch = in.getBool();
     myLDAimmediate = in.getBool();
 
     // Control Byte to update
-    myParameter = (uInt8) in.getByte();
+    limit = (uInt32) in.getInt();
+    for(i = 0; i < limit; ++i)
+      myParameter[i] = (uInt8) in.getByte();
 
     // The music mode counters for the data fetchers
     limit = (uInt32) in.getInt();
