@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartE7.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: CartE7.cxx 2199 2011-01-01 16:04:32Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -24,10 +24,12 @@
 #include "CartE7.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeE7::CartridgeE7(const uInt8* image)
+CartridgeE7::CartridgeE7(const uInt8* image, const Settings& settings)
+  : Cartridge(settings)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image, 16384);
+  createCodeAccessBase(16384 + 2048);
 
   // This cart can address a 1024 byte bank of RAM @ 0x1000
   // and 256 bytes @ 0x1800
@@ -48,9 +50,12 @@ CartridgeE7::~CartridgeE7()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE7::reset()
 {
-  // Initialize RAM with random values
-  for(uInt32 i = 0; i < 2048; ++i)
-    myRAM[i] = mySystem->randGenerator().next();
+  // Initialize RAM
+  if(mySettings.getBool("ramrandom"))
+    for(uInt32 i = 0; i < 2048; ++i)
+      myRAM[i] = mySystem->randGenerator().next();
+  else
+    memset(myRAM, 0, 2048);
 
   // Install some default banks for the RAM and first segment
   bankRAM(0);
@@ -70,23 +75,20 @@ void CartridgeE7::install(System& system)
   assert(((0x1400 & mask) == 0) && ((0x1800 & mask) == 0) &&
       ((0x1900 & mask) == 0) && ((0x1A00 & mask) == 0));
 
-  System::PageAccess access;
+  System::PageAccess access(0, 0, 0, this, System::PA_READ);
 
   // Set the page accessing methods for the hot spots
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
   for(uInt32 i = (0x1FE0 & ~mask); i < 0x2000; i += (1 << shift))
+  {
+    access.codeAccessBase = &myCodeAccessBase[8128];
     mySystem->setPageAccess(i >> shift, access);
+  }
 
   // Setup the second segment to always point to the last ROM slice
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READ;
   for(uInt32 j = 0x1A00; j < (0x1FE0U & ~mask); j += (1 << shift))
   {
     access.directPeekBase = &myImage[7 * 2048 + (j & 0x07FF)];
+    access.codeAccessBase = &myCodeAccessBase[7 * 2048 + (j & 0x07FF)];
     mySystem->setPageAccess(j >> shift, access);
   }
   myCurrentSlice[1] = 7;
@@ -174,78 +176,74 @@ void CartridgeE7::bankRAM(uInt16 bank)
   uInt16 shift = mySystem->pageShift();
 
   // Setup the page access methods for the current bank
-  System::PageAccess access;
+  System::PageAccess access(0, 0, 0, this, System::PA_WRITE);
 
   // Set the page accessing method for the 256 bytes of RAM writing pages
-  access.directPeekBase = 0;
-  access.device = this;
-  access.type = System::PA_WRITE;
   for(uInt32 j = 0x1800; j < 0x1900; j += (1 << shift))
   {
     access.directPokeBase = &myRAM[1024 + offset + (j & 0x00FF)];
+    access.codeAccessBase = &myCodeAccessBase[8192 + 1024 + offset + (j & 0x00FF)];
     mySystem->setPageAccess(j >> shift, access);
   }
 
   // Set the page accessing method for the 256 bytes of RAM reading pages
   access.directPokeBase = 0;
-  access.device = this;
   access.type = System::PA_READ;
   for(uInt32 k = 0x1900; k < 0x1A00; k += (1 << shift))
   {
     access.directPeekBase = &myRAM[1024 + offset + (k & 0x00FF)];
+    access.codeAccessBase = &myCodeAccessBase[8192 + 1024 + offset + (k & 0x00FF)];
     mySystem->setPageAccess(k >> shift, access);
   }
   myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeE7::bank(uInt16 slice)
+bool CartridgeE7::bank(uInt16 slice)
 { 
-  if(bankLocked()) return;
+  if(bankLocked()) return false;
 
   // Remember what bank we're in
   myCurrentSlice[0] = slice;
   uInt16 offset = slice << 11;
   uInt16 shift = mySystem->pageShift();
 
-  System::PageAccess access;
-
   // Setup the page access methods for the current bank
   if(slice != 7)
   {
+    System::PageAccess access(0, 0, 0, this, System::PA_READ);
+
     // Map ROM image into first segment
-    access.directPokeBase = 0;
-    access.device = this;
-    access.type = System::PA_READ;
     for(uInt32 address = 0x1000; address < 0x1800; address += (1 << shift))
     {
       access.directPeekBase = &myImage[offset + (address & 0x07FF)];
+      access.codeAccessBase = &myCodeAccessBase[offset + (address & 0x07FF)];
       mySystem->setPageAccess(address >> shift, access);
     }
   }
   else
   {
+    System::PageAccess access(0, 0, 0, this, System::PA_WRITE);
+
     // Set the page accessing method for the 1K slice of RAM writing pages
-    access.directPeekBase = 0;
-    access.device = this;
-    access.type = System::PA_WRITE;
     for(uInt32 j = 0x1000; j < 0x1400; j += (1 << shift))
     {
       access.directPokeBase = &myRAM[j & 0x03FF];
+      access.codeAccessBase = &myCodeAccessBase[8192 + (j & 0x03FF)];
       mySystem->setPageAccess(j >> shift, access);
     }
 
     // Set the page accessing method for the 1K slice of RAM reading pages
     access.directPokeBase = 0;
-    access.device = this;
     access.type = System::PA_READ;
     for(uInt32 k = 0x1400; k < 0x1800; k += (1 << shift))
     {
       access.directPeekBase = &myRAM[k & 0x03FF];
+      access.codeAccessBase = &myCodeAccessBase[8192 + (k & 0x03FF)];
       mySystem->setPageAccess(k >> shift, access);
     }
   }
-  myBankChanged = true;
+  return myBankChanged = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -300,13 +298,11 @@ const uInt8* CartridgeE7::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeE7::save(Serializer& out) const
 {
-  const string& cart = name();
-
   try
   {
     uInt32 i;
 
-    out.putString(cart);
+    out.putString(name());
 
     out.putInt(2);
     for(i = 0; i < 2; ++i)
@@ -331,11 +327,9 @@ bool CartridgeE7::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeE7::load(Serializer& in)
 {
-  const string& cart = name();
-
   try
   {
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     uInt32 i, limit;
