@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartMC.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: CartMC.cxx 2199 2011-01-01 16:04:32Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -28,10 +28,13 @@
 //        accurate the emulation is
 //        Bankchange and RAM modification cannot be completed until
 //        adequate test ROMs are available
+// TODO (2010-10-03) - support CodeAccessBase functionality somehow
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeMC::CartridgeMC(const uInt8* image, uInt32 size)
-  : mySlot3Locked(false)
+CartridgeMC::CartridgeMC(const uInt8* image, uInt32 size,
+                         const Settings& settings)
+  : Cartridge(settings),
+    mySlot3Locked(false)
 {
   // Make sure size is reasonable
   assert(size <= 131072);
@@ -51,9 +54,12 @@ CartridgeMC::~CartridgeMC()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeMC::reset()
 {
-  // Initialize RAM with random values
-  for(uInt32 i = 0; i < 32768; ++i)
-    myRAM[i] = mySystem->randGenerator().next();
+  // Initialize RAM
+  if(mySettings.getBool("ramrandom"))
+    for(uInt32 i = 0; i < 32768; ++i)
+      myRAM[i] = mySystem->randGenerator().next();
+  else
+    memset(myRAM, 0, 32768);
 
   myBankChanged = true;
 }
@@ -76,18 +82,12 @@ void CartridgeMC::install(System& system)
   // TODO: These TIA accesses may need to be chained, however, at this
   //       point Chris isn't sure if the hardware will allow it or not
   //
-  System::PageAccess access;
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
-  access.type = System::PA_READWRITE;
+  System::PageAccess access(0, 0, 0, this, System::PA_READWRITE);
+
   for(uInt32 i = 0x00; i < 0x40; i += (1 << shift))
     mySystem->setPageAccess(i >> shift, access);
 
   // Map the cartridge into the system
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = this;
   access.type = System::PA_READ;  // We don't yet indicate RAM areas
   for(uInt32 j = 0x1000; j < 0x2000; j += (1 << shift))
     mySystem->setPageAccess(j >> shift, access);
@@ -210,9 +210,10 @@ bool CartridgeMC::poke(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeMC::bank(uInt16 b)
+bool CartridgeMC::bank(uInt16 b)
 {
   // Doesn't support bankswitching in the normal sense
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -246,12 +247,10 @@ const uInt8* CartridgeMC::getImage(int& size) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeMC::save(Serializer& out) const
 {
-  uInt32 i;
-  const string& cart = name();
-
   try
   {
-    out.putString(cart);
+    uInt32 i;
+    out.putString(name());
 
     // The currentBlock array
     out.putInt(4);
@@ -275,14 +274,11 @@ bool CartridgeMC::save(Serializer& out) const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeMC::load(Serializer& in)
 {
-  uInt32 i;
-  const string& cart = name();
-
   try
   {
-    uInt32 limit;
+    uInt32 i, limit;
 
-    if(in.getString() != cart)
+    if(in.getString() != name())
       return false;
 
     // The currentBlock array
