@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Console.cxx 2155 2010-10-18 18:39:57Z stephena $
+// $Id: Console.cxx 2232 2011-05-24 16:04:48Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -132,7 +132,7 @@ Console::Console(OSystem* osystem, Cartridge* cart, const Properties& props)
     // The 'fastscbios' option must be changed before the system is reset
     bool fastscbios = myOSystem->settings().getBool("fastscbios");
     myOSystem->settings().setBool("fastscbios", true);
-    mySystem->reset();
+    mySystem->reset(true);  // autodetect in reset enabled
     int palCount = 0;
     for(int i = 0; i < 60; ++i)
     {
@@ -619,7 +619,8 @@ void Console::setControllers(const string& rommd5)
 
   // Also check if we should swap the paddles plugged into a jack
   bool swapPaddles = myProperties.get(Controller_SwapPaddles) == "YES";
-  Paddles::setMouseIsPaddle(-1);  // Reset to defaults
+  // Set default controller for mouse
+  Controller::setMouseIsController(0);
 
   // Construct left controller
   if(left == "BOOSTERGRIP")
@@ -634,9 +635,19 @@ void Console::setControllers(const string& rommd5)
   {
     myControllers[leftPort] = new Keyboard(Controller::Left, *myEvent, *mySystem);
   }
-  else if(left == "PADDLES")
+  else if(BSPF_startsWithIgnoreCase(left, "PADDLES"))
   {
-    myControllers[leftPort] = new Paddles(Controller::Left, *myEvent, *mySystem, swapPaddles);
+    Controller::setMouseIsController(swapPaddles ? 1 : 0);
+    bool swapAxis = false, swapDir = false;
+    if(left == "PADDLES_IAXIS")
+      swapAxis = true;
+    else if(left == "PADDLES_IDIR")
+      swapDir = true;
+    else if(left == "PADDLES_IAXDR")
+      swapAxis = swapDir = true;
+    myControllers[leftPort] =
+      new Paddles(Controller::Left, *myEvent, *mySystem,
+                  swapPaddles, swapAxis, swapDir);
   }
   else if(left == "TRACKBALL22")
   {
@@ -675,9 +686,18 @@ void Console::setControllers(const string& rommd5)
   {
     myControllers[rightPort] = new Keyboard(Controller::Right, *myEvent, *mySystem);
   }
-  else if(right == "PADDLES")
+  else if(BSPF_startsWithIgnoreCase(right, "PADDLES"))
   {
-    myControllers[rightPort] = new Paddles(Controller::Right, *myEvent, *mySystem, swapPaddles);
+    bool swapAxis = false, swapDir = false;
+    if(right == "PADDLES_IAXIS")
+      swapAxis = true;
+    else if(right == "PADDLES_IDIR")
+      swapDir = true;
+    else if(right == "PADDLES_IAXDR")
+      swapAxis = swapDir = true;
+    myControllers[rightPort] =
+      new Paddles(Controller::Right, *myEvent, *mySystem,
+                  swapPaddles, swapAxis, swapDir);
   }
   else if(right == "TRACKBALL22")
   {
