@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Control.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: Control.cxx 2228 2011-05-06 14:29:39Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -182,12 +182,23 @@ string Controller::about() const
   return name() + " in " + (myJack == Left ? "left port" : "right port");
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Controller::setMouseIsController(int number)
+{
+  ourControlNum = number;
+  if(ourControlNum < 0)       ourControlNum = 0;
+  else if (ourControlNum > 3) ourControlNum = 3;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const Int32 Controller::maximumResistance = 0x7FFFFFFF;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const Int32 Controller::minimumResistance = 0x00000000;
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Int32 Controller::ourControlNum = 0;
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Controller::Controller(const Controller& c)
   : myJack(c.myJack),
