@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FSNode.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: FSNode.cxx 2250 2011-06-09 14:00:30Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -39,9 +39,7 @@ FilesystemNode::FilesystemNode(AbstractFilesystemNode *realNode)
 FilesystemNode::FilesystemNode(const string& p)
 {
   AbstractFilesystemNode* tmp = 0;
-  if (p.empty() || p == ".")
-    tmp = AbstractFilesystemNode::makeCurrentDirectoryFileNode();
-  else if (p == "~")
+  if (p.empty() || p == "." || p == "~")
     tmp = AbstractFilesystemNode::makeHomeDirectoryFileNode();
   else
     tmp = AbstractFilesystemNode::makeFileNodePath(p);
@@ -124,17 +122,10 @@ FilesystemNode FilesystemNode::getParent() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string FilesystemNode::getPath() const
+string FilesystemNode::getPath(bool fqn) const
 {
   assert(_realNode);
-  return _realNode->getPath();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string FilesystemNode::getRelativePath() const
-{
-  assert(_realNode);
-  return _realNode->getRelativePath();
+  return _realNode->getPath(fqn);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
