@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBuffer.cxx 2169 2010-11-07 16:19:26Z stephena $
+// $Id: FrameBuffer.cxx 2249 2011-06-07 13:40:59Z stephena $
 //============================================================================
 
 #include <algorithm>
@@ -67,7 +67,8 @@ FrameBuffer::~FrameBuffer(void)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-FBInitStatus FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height)
+FBInitStatus FrameBuffer::initialize(const string& title,
+                                     uInt32 width, uInt32 height)
 {
   ostringstream buf;
 
@@ -286,7 +287,6 @@ void FrameBuffer::update()
 
     default:
       return;
-      break;
   }
 
   // Draw any pending messages
@@ -743,21 +743,21 @@ cerr << "New mode:" << endl
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBuffer::setCursorState()
 {
-  if(fullScreen())
-    grabMouse(true);
-  else
-    grabMouse(myOSystem->settings().getBool("grabmouse"));
+  // Always grab mouse in fullscreen or during emulation (if enabled),
+  // and don't show the cursor during emulation
+  bool emulation =
+      myOSystem->eventHandler().state() == EventHandler::S_EMULATE;
+  grabMouse(fullScreen() ||
+    (emulation && myOSystem->settings().getBool("grabmouse")));
+  showCursor(!emulation);
+}
 
-  switch(myOSystem->eventHandler().state())
-  {
-    case EventHandler::S_EMULATE:
-    case EventHandler::S_PAUSE:
-      showCursor(false);
-      break;
-    default:
-      showCursor(true);
-      break;
-  }
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBuffer::toggleGrabMouse()
+{
+  bool state = myOSystem->settings().getBool("grabmouse");
+  myOSystem->settings().setBool("grabmouse", !state);
+  setCursorState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -850,7 +850,7 @@ void FrameBuffer::setWindowIcon()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8 FrameBuffer::getPhosphor(uInt8 c1, uInt8 c2)
+uInt8 FrameBuffer::getPhosphor(uInt8 c1, uInt8 c2) const
 {
   if(c2 > c1)
     BSPF_swap(c1, c2);
