@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: OSystem.cxx 2168 2010-11-04 19:29:12Z stephena $
+// $Id: OSystem.cxx 2251 2011-06-09 20:50:19Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -325,19 +325,19 @@ void OSystem::setConfigPaths()
   if(s == "") s = myBaseDir + "stella.cht";
   node = FilesystemNode(s);
   myCheatFile = node.getPath();
-  mySettings->setString("cheatfile", node.getRelativePath());
+  mySettings->setString("cheatfile", node.getPath(false));
 
   s = mySettings->getString("palettefile");
   if(s == "") s = myBaseDir + "stella.pal";
   node = FilesystemNode(s);
   myPaletteFile = node.getPath();
-  mySettings->setString("palettefile", node.getRelativePath());
+  mySettings->setString("palettefile", node.getPath(false));
 
   s = mySettings->getString("propsfile");
   if(s == "") s = myBaseDir + "stella.pro";
   node = FilesystemNode(s);
   myPropertiesFile = node.getPath();
-  mySettings->setString("propsfile", node.getRelativePath());
+  mySettings->setString("propsfile", node.getPath(false));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -479,7 +479,7 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
   // Do a little error checking; it shouldn't be necessary
   if(myConsole) deleteConsole();
 
-  bool retval = false, showmessage = false;
+  bool showmessage = false;
 
   // If a blank ROM has been given, we reload the current one (assuming one exists)
   if(romfile == "")
@@ -546,25 +546,24 @@ bool OSystem::createConsole(const string& romfile, const string& md5sum)
     resetLoopTiming();
 
     myFrameBuffer->setCursorState();
-    retval = true;
+
+    // Also check if certain virtual buttons should be held down
+    // These must be checked each time a new console is being created
+    if(mySettings->getBool("holdreset"))
+      myEventHandler->handleEvent(Event::ConsoleReset, 1);
+    if(mySettings->getBool("holdselect"))
+      myEventHandler->handleEvent(Event::ConsoleSelect, 1);
+    if(mySettings->getBool("holdbutton0"))
+      myEventHandler->handleEvent(Event::JoystickZeroFire1, 1);
+
+    return true;
   }
   else
   {
     buf << "ERROR: Couldn't create console for " << myRomFile << endl;
     logMessage(buf.str(), 0);
-    retval = false;
+    return false;
   }
-
-  // Also check if certain virtual buttons should be held down
-  // These must be checked each time a new console is being created
-  if(mySettings->getBool("holdreset"))
-    myEventHandler->handleEvent(Event::ConsoleReset, 1);
-  if(mySettings->getBool("holdselect"))
-    myEventHandler->handleEvent(Event::ConsoleSelect, 1);
-  if(mySettings->getBool("holdbutton0"))
-    myEventHandler->handleEvent(Event::JoystickZeroFire1, 1);
-
-  return retval;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -643,9 +642,16 @@ string OSystem::MD5FromFile(const string& filename)
 void OSystem::logMessage(const string& message, uInt8 level)
 {
   if(level == 0)
-    cerr << message;
+  {
+    cout << message << flush;
+    myLogMessages += message;
+  }
   else if(level <= (uInt8)mySettings->getInt("showinfo"))
-    cout << message;
+  {
+    if(1) // TODO - messages should be output to console
+      cout << message << flush;
+    myLogMessages += message;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -806,6 +812,13 @@ uInt8* OSystem::openROM(string file, string& md5, uInt32& size)
     gzclose(f);
   }
 
+  // Zero-byte files should be automatically discarded
+  if(size == 0)
+  {
+    delete[] image;
+    return (uInt8*) 0;
+  }
+
   // If we get to this point, we know we have a valid file to open
   // Now we make sure that the file has a valid properties entry
   // To save time, only generate an MD5 if we really need one
@@ -861,14 +874,14 @@ void OSystem::validatePath(const string& setting, const string& partialpath,
 {
   const string& s = mySettings->getString(setting) != "" ?
     mySettings->getString(setting) : myBaseDir + partialpath;
-  FilesystemNode node = FilesystemNode(s);
+  FilesystemNode node(s);
   if(!node.isDirectory())
   {
     AbstractFilesystemNode::makeDir(s);
     node = FilesystemNode(node.getPath());
   }
   fullpath = node.getPath();
-  mySettings->setString(setting, node.getRelativePath());
+  mySettings->setString(setting, node.getPath(false));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
