@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Serializer.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: Serializer.cxx 2199 2011-01-01 16:04:32Z stephena $
 //============================================================================
 
 #include <fstream>
@@ -152,17 +152,15 @@ string Serializer::getString(void)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Serializer::getBool(void)
 {
-  bool result = false;
-
   char b = getByte();
   if(b == (char)TruePattern)
-    result = true;
+    return true;
   else if(b == (char)FalsePattern)
-    result = false;
+    return false;
   else
     throw "Serializer::getBool() data corruption";
 
-  return result;
+  return false;  // to stop compiler from complaining
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
