@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TIA.cxx 2012 2010-04-14 22:35:46Z stephena $
+// $Id: TIA.cxx 2199 2011-01-01 16:04:32Z stephena $
 //============================================================================
 
 #include <cassert>
@@ -23,6 +23,10 @@
 
 #include "bspf.hxx"
 
+#ifdef DEBUGGER_SUPPORT
+  #include "CartDebug.hxx"
+#endif
+
 #include "Console.hxx"
 #include "Control.hxx"
 #include "Device.hxx"
@@ -42,6 +46,9 @@ TIA::TIA(Console& console, Sound& sound, Settings& settings)
   : myConsole(console),
     mySound(sound),
     mySettings(settings),
+    myFrameWidth(160),
+    myFrameYStart(34),
+    myFrameHeight(210),
     myMaximumNumberOfScanlines(262),
     myColorLossEnabled(false),
     myPartialFrameFlag(false),
@@ -169,6 +176,7 @@ void TIA::reset()
 
   myDumpEnabled = false;
   myDumpDisabledCycle = 0;
+  myINPT4 = myINPT5 = 0x80;
 
   // Should undriven pins be randomly driven high or low?
   myTIAPinsDriven = mySettings.getBool("tiadriven");
@@ -205,6 +213,13 @@ void TIA::reset()
     myMaximumNumberOfScanlines = 342;
   }
 
+  myCurrentP0Mask = &TIATables::PxMask[0][0][0][0];
+  myCurrentP1Mask = &TIATables::PxMask[0][0][0][0];
+  myCurrentM0Mask = &TIATables::MxMask[0][0][0][0];
+  myCurrentM1Mask = &TIATables::MxMask[0][0][0][0];
+  myCurrentBLMask = &TIATables::BLMask[0][0][0];
+  myCurrentPFMask = TIATables::PFMask[0];
+
   // Recalculate the size of the display
   frameReset();
 }
@@ -218,15 +233,6 @@ void TIA::frameReset()
   // Reset pixel pointer and drawing flag
   myFramePointer = myCurrentFrameBuffer;
 
-  // Make sure all these are within bounds
-  myFrameWidth  = 160;
-  myFrameYStart = atoi(myConsole.properties().get(Display_YStart).c_str());
-  if(myFrameYStart < 0)       myFrameYStart = 0;
-  else if(myFrameYStart > 64) myFrameYStart = 64;
-  myFrameHeight = atoi(myConsole.properties().get(Display_Height).c_str());
-  if(myFrameHeight < 210)      myFrameHeight = 210;
-  else if(myFrameHeight > 256) myFrameHeight = 256;
-
   // Calculate color clock offsets for starting and stopping frame drawing
   // Note that although we always start drawing at scanline zero, the
   // framebuffer that is exposed outside the class actually starts at 'ystart'
@@ -291,11 +297,7 @@ void TIA::install(System& system, Device& device)
   mySystem->resetCycles();
 
   // All accesses are to the given device
-  System::PageAccess access;
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  access.device = &device;
-  access.type = System::PA_READWRITE;
+  System::PageAccess access(0, 0, 0, &device, System::PA_READWRITE);
 
   // We're installing in a 2600 system
   for(uInt32 i = 0; i < 8192; i += (1 << shift))
@@ -1161,46 +1163,42 @@ uInt8 TIA::peek(uInt16 addr)
 
   // If pins are undriven, we start with the last databus value
   // Otherwise, there is some randomness injected into the mix
-  uInt8 value = myTIAPinsDriven ? mySystem->getDataBusState(0xFF) :
-                                  mySystem->getDataBusState();
+  // In either case, we start out with D7 and D6 disabled (the only
+  // valid bits in a TIA read), and selectively enable them
+  uInt8 value = 0x3F & (!myTIAPinsDriven ? mySystem->getDataBusState() :
+                        mySystem->getDataBusState(0xFF));
   uInt16 collision = myCollision & (uInt16)myCollisionEnabledMask;
 
   switch(addr & 0x000f)
   {
     case CXM0P:
-      value = (value & 0x3F) |
-              ((collision & Cx_M0P1) ? 0x80 : 0x00) |
-              ((collision & Cx_M0P0) ? 0x40 : 0x00);
+      value |= ((collision & Cx_M0P1) ? 0x80 : 0x00) |
+               ((collision & Cx_M0P0) ? 0x40 : 0x00);
       break;
 
     case CXM1P:
-      value = (value & 0x3F) |
-              ((collision & Cx_M1P0) ? 0x80 : 0x00) |
-              ((collision & Cx_M1P1) ? 0x40 : 0x00);
+      value |= ((collision & Cx_M1P0) ? 0x80 : 0x00) |
+               ((collision & Cx_M1P1) ? 0x40 : 0x00);
       break;
 
     case CXP0FB:
-      value = (value & 0x3F) |
-              ((collision & Cx_P0PF) ? 0x80 : 0x00) |
-              ((collision & Cx_P0BL) ? 0x40 : 0x00);
+      value |= ((collision & Cx_P0PF) ? 0x80 : 0x00) |
+               ((collision & Cx_P0BL) ? 0x40 : 0x00);
       break;
 
     case CXP1FB:
-      value = (value & 0x3F) |
-              ((collision & Cx_P1PF) ? 0x80 : 0x00) |
-              ((collision & Cx_P1BL) ? 0x40 : 0x00);
+      value |= ((collision & Cx_P1PF) ? 0x80 : 0x00) |
+               ((collision & Cx_P1BL) ? 0x40 : 0x00);
       break;
 
     case CXM0FB:
-      value = (value & 0x3F) |
-              ((collision & Cx_M0PF) ? 0x80 : 0x00) |
-              ((collision & Cx_M0BL) ? 0x40 : 0x00);
+      value |= ((collision & Cx_M0PF) ? 0x80 : 0x00) |
+               ((collision & Cx_M0BL) ? 0x40 : 0x00);
       break;
 
     case CXM1FB:
-      value = (value & 0x3F) |
-              ((collision & Cx_M1PF) ? 0x80 : 0x00) |
-              ((collision & Cx_M1BL) ? 0x40 : 0x00);
+      value |= ((collision & Cx_M1PF) ? 0x80 : 0x00) |
+               ((collision & Cx_M1BL) ? 0x40 : 0x00);
       break;
 
     case CXBLPF:
@@ -1208,9 +1206,8 @@ uInt8 TIA::peek(uInt16 addr)
       break;
 
     case CXPPMM:
-      value = (value & 0x3F) |
-              ((collision & Cx_P0P1) ? 0x80 : 0x00) |
-              ((collision & Cx_M0M1) ? 0x40 : 0x00);
+      value |= ((collision & Cx_P0P1) ? 0x80 : 0x00) |
+               ((collision & Cx_M0M1) ? 0x40 : 0x00);
       break;
 
     case INPT0:
@@ -1234,16 +1231,26 @@ uInt8 TIA::peek(uInt16 addr)
       break;
 
     case INPT4:
-      value = (value & 0x7F) |
-        (myConsole.controller(Controller::Left).read(Controller::Six) ? 0x80 : 0x00);
+    {
+      uInt8 button = (myConsole.controller(Controller::Left).read(Controller::Six) ? 0x80 : 0x00);
+      myINPT4 = (myVBLANK & 0x40) ? (myINPT4 & button) : button;
+
+      value = (value & 0x7F) | myINPT4;
       break;
+    }
 
     case INPT5:
-      value = (value & 0x7F) |
-        (myConsole.controller(Controller::Right).read(Controller::Six) ? 0x80 : 0x00);
+    {
+      uInt8 button = (myConsole.controller(Controller::Right).read(Controller::Six) ? 0x80 : 0x00);
+      myINPT5 = (myVBLANK & 0x40) ? (myINPT5 & button) : button;
+
+      value = (value & 0x7F) | myINPT5;
       break;
+    }
 
     default:
+      // This shouldn't happen, but if it does, we essentially just
+      // return the last databus value with bits D6 and D7 zeroed out
       break;
   }
   return value;
@@ -1314,6 +1321,10 @@ bool TIA::poke(uInt16 addr, uInt8 value)
         myDumpDisabledCycle = mySystem->cycles();
       }
 
+      // Are the latches for I4 and I5 being set?
+      if (!(myVBLANK & 0x40))
+        myINPT4 = myINPT5 = 0x80;
+
 #if 0 // TODO - this isn't yet complete
       // Check for the first scanline at which VBLANK is disabled.
       // Usually, this will be the first scanline to start drawing.
@@ -1461,6 +1472,11 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects |= PFBit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::PGFX);
+    #endif
       break;
     }
 
@@ -1473,6 +1489,11 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects |= PFBit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::PGFX);
+    #endif
       break;
     }
 
@@ -1485,6 +1506,11 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects |= PFBit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::PGFX);
+    #endif
       break;
     }
 
@@ -1720,6 +1746,11 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects &= ~P1Bit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::GFX);
+    #endif
       break;
     }
 
@@ -1758,6 +1789,11 @@ bool TIA::poke(uInt16 addr, uInt8 value)
       else
         myEnabledObjects &= ~BLBit;
 
+    #ifdef DEBUGGER_SUPPORT
+      uInt16 dataAddr = mySystem->m6502().lastDataAddressForPoke();
+      if(dataAddr)
+        mySystem->setAccessFlags(dataAddr, CartDebug::GFX);
+    #endif
       break;
     }
 
