@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EventMappingWidget.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: EventMappingWidget.cxx 2199 2011-01-01 16:04:32Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -27,8 +27,10 @@
 #include "EventHandler.hxx"
 #include "Event.hxx"
 #include "OSystem.hxx"
+#include "EditTextWidget.hxx"
 #include "StringListWidget.hxx"
 #include "Widget.hxx"
+#include "ComboDialog.hxx"
 
 #include "EventMappingWidget.hxx"
 
@@ -64,12 +66,7 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
                                  "Map", kStartMapCmd);
   myMapButton->setTarget(this);
   addFocusWidget(myMapButton);
-  ypos += lineHeight + 10;
-  myEraseButton = new ButtonWidget(boss, font, xpos, ypos,
-                                   buttonWidth, buttonHeight,
-                                   "Erase", kEraseCmd);
-  myEraseButton->setTarget(this);
-  addFocusWidget(myEraseButton);
+
   ypos += lineHeight + 10;
   myCancelMapButton = new ButtonWidget(boss, font, xpos, ypos,
                                        buttonWidth, buttonHeight,
@@ -77,18 +74,47 @@ EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
   myCancelMapButton->setTarget(this);
   myCancelMapButton->clearFlags(WIDGET_ENABLED);
   addFocusWidget(myCancelMapButton);
-  ypos += lineHeight + 30;
-  myDefaultsButton = new ButtonWidget(boss, font, xpos, ypos,
-                                      buttonWidth, buttonHeight,
-                                      "Defaults", kDefaultsCmd);
-  myDefaultsButton->setTarget(this);
-  addFocusWidget(myDefaultsButton);
+
+  ypos += lineHeight + 20;
+  myEraseButton = new ButtonWidget(boss, font, xpos, ypos,
+                                   buttonWidth, buttonHeight,
+                                   "Erase", kEraseCmd);
+  myEraseButton->setTarget(this);
+  addFocusWidget(myEraseButton);
+
+  ypos += lineHeight + 10;
+  myResetButton = new ButtonWidget(boss, font, xpos, ypos,
+                                   buttonWidth, buttonHeight,
+                                   "Reset", kResetCmd);
+  myResetButton->setTarget(this);
+  addFocusWidget(myResetButton);
+
+  if(mode == kEmulationMode)
+  {
+    ypos += lineHeight + 20;
+    myComboButton = new ButtonWidget(boss, font, xpos, ypos,
+                                     buttonWidth, buttonHeight,
+                                     "Combo", kComboCmd);
+    myComboButton->setTarget(this);
+    addFocusWidget(myComboButton);
+
+    myComboDialog = new ComboDialog(boss, font,
+      instance().eventHandler().getComboList(mode));
+  }
+  else
+    myComboButton = NULL;
 
   // Show message for currently selected event
-  xpos = 10;  ypos = 5 + myActionsList->getHeight() + 3;
-  myKeyMapping  = new StaticTextWidget(boss, font, xpos, ypos, _w - 20, fontHeight,
-                                       "Action: ", kTextAlignLeft);
-  myKeyMapping->setFlags(WIDGET_CLEARBG);
+  xpos = 10;  ypos = 5 + myActionsList->getHeight() + 5;
+  StaticTextWidget* t;
+  t = new StaticTextWidget(boss, font, xpos, ypos, font.getStringWidth("Action:"),
+                           fontHeight, "Action:", kTextAlignLeft);
+  t->setFlags(WIDGET_CLEARBG);
+
+  myKeyMapping = new EditTextWidget(boss, font, xpos + t->getWidth() + 5, ypos,
+                                    _w - xpos - t->getWidth() - 15, lineHeight, "");
+  myKeyMapping->setEditable(false);
+  myKeyMapping->clearFlags(WIDGET_RETAIN_FOCUS);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -107,7 +133,8 @@ void EventMappingWidget::loadConfig()
 
   // Make sure remapping is turned off, just in case the user didn't properly
   // exit last time
-  stopRemapping();
+  if(myRemapStatus)
+    stopRemapping();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -115,6 +142,13 @@ void EventMappingWidget::saveConfig()
 {
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventMappingWidget::setDefaults()
+{
+  instance().eventHandler().setDefaultMapping(Event::NoType, myEventMode);
+  drawKeyMapping();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventMappingWidget::startRemapping()
 {
@@ -124,20 +158,19 @@ void EventMappingWidget::startRemapping()
   // Set the flags for the next event that arrives
   myRemapStatus = true;
 
+  // Reset all previous events for determining correct axis/hat values
+  myLastStick = myLastAxis = myLastHat = myLastValue = -1;
+
   // Disable all other widgets while in remap mode, except enable 'Cancel'
-  myActionsList->setEnabled(false);
-  myMapButton->setEnabled(false);
-  myEraseButton->setEnabled(false);
-  myDefaultsButton->setEnabled(false);
-  myCancelMapButton->setEnabled(true);
+  enableButtons(false);
 
   // And show a message indicating which key is being remapped
   ostringstream buf;
   buf << "Select action for '"
       << instance().eventHandler().actionAtIndex(myActionSelected, myEventMode)
-      << "' event";	 	
+      << "' event";
   myKeyMapping->setTextColor(kTextColorEm);
-  myKeyMapping->setLabel(buf.str());
+  myKeyMapping->setEditString(buf.str());
 
   // Make sure that this widget receives all raw data, before any
   // pre-processing occurs
@@ -157,26 +190,33 @@ void EventMappingWidget::eraseRemapping()
   drawKeyMapping();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventMappingWidget::resetRemapping()
+{
+  if(myActionSelected < 0)
+    return;
+
+  Event::Type event =
+    instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+  instance().eventHandler().setDefaultMapping(event, myEventMode);
+
+  drawKeyMapping();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventMappingWidget::stopRemapping()
 {
   // Turn off remap mode
   myRemapStatus = false;
 
+  // Reset all previous events for determining correct axis/hat values
+  myLastStick = myLastAxis = myLastHat = myLastValue = -1;
+
   // And re-enable all the widgets
-  myActionsList->setEnabled(true);
-  myMapButton->setEnabled(false);
-  myEraseButton->setEnabled(false);
-  myDefaultsButton->setEnabled(true);
-  myCancelMapButton->setEnabled(false);
+  enableButtons(true);
 
   // Make sure the list widget is in a known state
-  if(myActionSelected >= 0)
-  {
-    drawKeyMapping();
-    myMapButton->setEnabled(true);
-    myEraseButton->setEnabled(true);
-  }
+  drawKeyMapping();
 
   // Widget is now free to process events normally
   myActionsList->clearFlags(WIDGET_WANTS_RAWDATA);
@@ -187,11 +227,25 @@ void EventMappingWidget::drawKeyMapping()
 {
   if(myActionSelected >= 0)
   {
-    ostringstream buf;
-    buf << "Action: "
-        << instance().eventHandler().keyAtIndex(myActionSelected, myEventMode);
     myKeyMapping->setTextColor(kTextColor);
-    myKeyMapping->setLabel(buf.str());
+    myKeyMapping->setEditString(instance().eventHandler().keyAtIndex(myActionSelected, myEventMode));
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventMappingWidget::enableButtons(bool state)
+{
+  myActionsList->setEnabled(state);
+  myMapButton->setEnabled(state);
+  myCancelMapButton->setEnabled(!state);
+  myEraseButton->setEnabled(state);
+  myResetButton->setEnabled(state);
+  if(myComboButton)
+  {
+    Event::Type e =
+      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+
+    myComboButton->setEnabled(state && e >= Event::Combo1 && e <= Event::Combo16);
   }
 }
 
@@ -226,35 +280,69 @@ void EventMappingWidget::handleJoyDown(int stick, int button)
 void EventMappingWidget::handleJoyAxis(int stick, int axis, int value)
 {
   // Remap joystick axes in remap mode
+  // There are two phases to detection:
+  //   First, detect an axis 'on' event
+  //   Then, detect the same axis 'off' event
   if(myRemapStatus && myActionSelected >= 0)
   {
-    Event::Type event =
-      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance().eventHandler().addJoyAxisMapping(event, myEventMode,
-                                                    stick, axis, value))
-      stopRemapping();
+    // Detect the first axis event that represents 'on'
+    if(myLastStick == -1 && myLastAxis == -1 && value != 0)
+    {
+      myLastStick = stick;
+      myLastAxis = axis;
+      myLastValue = value;
+    }
+    // Detect the first axis event that matches a previously set
+    // stick and axis, but turns the axis 'off'
+    else if(myLastStick == stick && axis == myLastAxis && value == 0)
+    {
+      value = myLastValue;
+
+      Event::Type event =
+        instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+      if(instance().eventHandler().addJoyAxisMapping(event, myEventMode,
+                                                     stick, axis, value))
+        stopRemapping();
+    }
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool EventMappingWidget::handleJoyHat(int stick, int hat, int value)
 {
-  bool result = false;
-
   // Remap joystick hats in remap mode
+  // There are two phases to detection:
+  //   First, detect a hat direction event
+  //   Then, detect the same hat 'center' event
   if(myRemapStatus && myActionSelected >= 0)
   {
-    Event::Type event =
-      instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
-    if(instance().eventHandler().addJoyHatMapping(event, myEventMode,
-                                                   stick, hat, value))
+    // Detect the first hat event that represents a valid direction
+    if(myLastStick == -1 && myLastHat == -1 && value != EVENT_HATCENTER)
     {
-      stopRemapping();
-      result = true;
+      myLastStick = stick;
+      myLastHat = hat;
+      myLastValue = value;
+
+      return true;
+    }
+    // Detect the first hat event that matches a previously set
+    // stick and hat, but centers the hat
+    else if(myLastStick == stick && hat == myLastHat && value == EVENT_HATCENTER)
+    {
+      value = myLastValue;
+
+      Event::Type event =
+        instance().eventHandler().eventAtIndex(myActionSelected, myEventMode);
+      if(instance().eventHandler().addJoyHatMapping(event, myEventMode,
+                                                    stick, hat, value))
+      {
+        stopRemapping();
+        return true;
+      }
     }
   }
 
-  return result;
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -268,9 +356,7 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
       {
         myActionSelected = myActionsList->getSelected();
         drawKeyMapping();
-        myMapButton->setEnabled(true);
-        myEraseButton->setEnabled(true);
-        myCancelMapButton->setEnabled(false);
+        enableButtons(true);
       }
       break;
 
@@ -288,17 +374,23 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
       startRemapping();
       break;
 
-    case kEraseCmd:
-      eraseRemapping();
-      break;
-
     case kStopMapCmd:
       stopRemapping();
       break;
 
-    case kDefaultsCmd:
-      instance().eventHandler().setDefaultMapping(myEventMode);
-      drawKeyMapping();
+    case kEraseCmd:
+      eraseRemapping();
+      break;
+
+    case kResetCmd:
+      resetRemapping();
+      break;
+
+    case kComboCmd:
+      if(myComboDialog)
+        myComboDialog->show(
+          instance().eventHandler().eventAtIndex(myActionSelected, myEventMode),
+          instance().eventHandler().actionAtIndex(myActionSelected, myEventMode));
       break;
   }
 }
