@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: InputDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: InputDialog.cxx 2245 2011-06-02 20:53:01Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -36,19 +36,20 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 InputDialog::InputDialog(OSystem* osystem, DialogContainer* parent,
-                         const GUI::Font& font)
+                         const GUI::Font& font, int max_w, int max_h)
   : Dialog(osystem, parent, 0, 0, 0, 0)
 {
   const int lineHeight   = font.getLineHeight(),
             fontWidth    = font.getMaxCharWidth(),
+            buttonWidth  = font.getStringWidth("Defaults") + 20,
             buttonHeight = font.getLineHeight() + 4;
   const int vBorder = 4;
   int xpos, ypos, tabID;
   WidgetArray wid;
 
   // Set real dimensions
-  _w = 42 * fontWidth + 10;
-  _h = 12 * (lineHeight + 4) + 10;
+  _w = BSPF_min(50 * fontWidth + 10, max_w);
+  _h = BSPF_min(13 * (lineHeight + 4) + 10, max_h);
 
   // The tab widget
   xpos = 2; ypos = vBorder;
@@ -77,15 +78,19 @@ InputDialog::InputDialog(OSystem* osystem, DialogContainer* parent,
   myTab->setParentWidget(tabID, myMenuEventMapper);
   addToFocusList(myMenuEventMapper->getFocusList(), tabID);
 
-  // 3) Virtual device support
-  addVDeviceTab(font);
+  // 3) Devices & ports
+  addDevicePortTab(font);
 
   // Finalize the tabs, and activate the first tab
   myTab->activateTabs();
   myTab->setActiveTab(0);
 
-  // Add OK and Cancel buttons
+  // Add Defaults, OK and Cancel buttons
   wid.clear();
+  ButtonWidget* b;
+  b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Defaults", kDefaultsCmd);
+  wid.push_back(b);
   addOKCancelBGroup(wid, font);
   addBGroupToFocusList(wid);
 }
@@ -96,16 +101,17 @@ InputDialog::~InputDialog()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void InputDialog::addVDeviceTab(const GUI::Font& font)
+void InputDialog::addDevicePortTab(const GUI::Font& font)
 {
   const int lineHeight = font.getLineHeight(),
+            fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight();
   int xpos, ypos, lwidth, pwidth, tabID;
   WidgetArray wid;
   StringMap items;
 
-  // Virtual device/ports
-  tabID = myTab->addTab("Virtual Devs");
+  // Devices/ports
+  tabID = myTab->addTab("Devices & Ports");
 
   // Stelladaptor mappings
   xpos = 5;  ypos = 5;
@@ -134,31 +140,43 @@ void InputDialog::addVDeviceTab(const GUI::Font& font)
                                   fwidth, fontHeight, "");
   wid.push_back(myAVoxPort);
 
-  lwidth = font.getStringWidth("Digital paddle speed: ");
+  lwidth = font.getStringWidth("Digital paddle sensitivity: ");
   pwidth = font.getMaxCharWidth() * 8;
 
   // Add joystick deadzone setting
-  ypos += 2*lineHeight;
+  ypos += lineHeight + 5;
   myDeadzone = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                                "Joystick deadzone: ", lwidth, kDeadzoneChanged);
+                                "Joystick deadzone size: ", lwidth, kDeadzoneChanged);
   myDeadzone->setMinValue(0); myDeadzone->setMaxValue(29);
   xpos += myDeadzone->getWidth() + 5;
-  myDeadzoneLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
-                                         "", kTextAlignLeft);
+  myDeadzoneLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 5*fontWidth,
+                                         lineHeight, "", kTextAlignLeft);
   myDeadzoneLabel->setFlags(WIDGET_CLEARBG);
   wid.push_back(myDeadzone);
 
-  // Add paddle speed
+  // Add paddle speed (digital emulation)
   xpos = 5;  ypos += lineHeight + 3;
-  myPaddleSpeed = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
-                                   "Digital paddle speed: ",
-                                   lwidth, kPSpeedChanged);
-  myPaddleSpeed->setMinValue(1); myPaddleSpeed->setMaxValue(15);
-  xpos += myPaddleSpeed->getWidth() + 5;
-  myPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
-                                       "", kTextAlignLeft);
-  myPaddleLabel->setFlags(WIDGET_CLEARBG);
-  wid.push_back(myPaddleSpeed);
+  myDPaddleSpeed = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                                   "Digital paddle sensitivity: ",
+                                   lwidth, kDPSpeedChanged);
+  myDPaddleSpeed->setMinValue(1); myDPaddleSpeed->setMaxValue(10);
+  xpos += myDPaddleSpeed->getWidth() + 5;
+  myDPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
+                                        "", kTextAlignLeft);
+  myDPaddleLabel->setFlags(WIDGET_CLEARBG);
+  wid.push_back(myDPaddleSpeed);
+
+  // Add paddle speed (mouse emulation)
+  xpos = 5;  ypos += lineHeight + 3;
+  myMPaddleSpeed = new SliderWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                                   "Mouse paddle sensitivity: ",
+                                   lwidth, kMPSpeedChanged);
+  myMPaddleSpeed->setMinValue(1); myMPaddleSpeed->setMaxValue(15);
+  xpos += myMPaddleSpeed->getWidth() + 5;
+  myMPaddleLabel = new StaticTextWidget(myTab, font, xpos, ypos+1, 24, lineHeight,
+                                        "", kTextAlignLeft);
+  myMPaddleSpeed->setFlags(WIDGET_CLEARBG);
+  wid.push_back(myMPaddleSpeed);
 
   // Add 'allow all 4 directions' for joystick
   xpos = 10;  ypos += 2*lineHeight;
@@ -172,6 +190,15 @@ void InputDialog::addVDeviceTab(const GUI::Font& font)
                      "Use mouse as a controller");
   wid.push_back(myMouseEnabled);
 
+  // Grab mouse (in windowed mode)	 
+  ypos += lineHeight + 4;
+  myGrabMouse = new CheckboxWidget(myTab, font, xpos, ypos,
+	                "Grab mouse in emulation mode");
+  wid.push_back(myGrabMouse);
+#ifndef WINDOWED_SUPPORT
+  myGrabMouse->clearFlags(WIDGET_ENABLED);	 
+#endif
+
   // Add items for virtual device ports
   addToFocusList(wid, tabID);
 }
@@ -189,15 +216,19 @@ void InputDialog::loadConfig()
 
   // Joystick deadzone
   myDeadzone->setValue(instance().settings().getInt("joydeadzone"));
-  myDeadzoneLabel->setLabel(instance().settings().getString("joydeadzone"));
+  myDeadzoneLabel->setValue(Joystick::deadzone());
 
   // Mouse/paddle enabled
-  bool usemouse = instance().settings().getBool("usemouse");
-  myMouseEnabled->setState(usemouse);
+  myMouseEnabled->setState(instance().settings().getBool("usemouse"));
 
-  // Paddle speed
-  myPaddleSpeed->setValue(instance().settings().getInt("pspeed"));
-  myPaddleLabel->setLabel(instance().settings().getString("pspeed"));
+  // Grab mouse
+  myGrabMouse->setState(instance().settings().getBool("grabmouse"));
+
+  // Paddle speed (digital and mouse)
+  myDPaddleSpeed->setValue(instance().settings().getInt("dsense"));
+  myDPaddleLabel->setLabel(instance().settings().getString("dsense"));
+  myMPaddleSpeed->setValue(instance().settings().getInt("msense"));
+  myMPaddleLabel->setLabel(instance().settings().getString("msense"));
 
   // AtariVox serial port
   myAVoxPort->setEditString(instance().settings().getString("avoxport"));
@@ -224,12 +255,19 @@ void InputDialog::saveConfig()
   // Mouse/paddle enabled
   bool usemouse = myMouseEnabled->getState();
   instance().settings().setBool("usemouse", usemouse);
-  instance().eventHandler().setPaddleMode(usemouse ? 0 : -1);
+  instance().eventHandler().setMouseControllerMode(usemouse ? 0 : -1);
 
-  // Paddle speed
-  int speed = myPaddleSpeed->getValue();
-  instance().settings().setInt("pspeed", speed);
-  Paddles::setDigitalSpeed(speed);
+  // Grab mouse	 
+  instance().settings().setBool("grabmouse", myGrabMouse->getState());	 
+  instance().frameBuffer().setCursorState();
+
+  // Paddle speed (digital and mouse)
+  int sensitivity = myDPaddleSpeed->getValue();
+  instance().settings().setInt("dsense", sensitivity);
+  Paddles::setDigitalSensitivity(sensitivity);
+  sensitivity = myMPaddleSpeed->getValue();
+  instance().settings().setInt("msense", sensitivity);
+  Paddles::setMouseSensitivity(sensitivity);
 
   // AtariVox serial port
   instance().settings().setString("avoxport", myAVoxPort->getEditString());
@@ -240,6 +278,56 @@ void InputDialog::saveConfig()
   instance().eventHandler().allowAllDirections(allowall4);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void InputDialog::setDefaults()
+{
+  switch(myTab->getActiveTab())
+  {
+    case 0:  // Emulation events
+      myEmulEventMapper->setDefaults();
+      break;
+
+    case 1:  // UI events
+      myMenuEventMapper->setDefaults();
+      break;
+
+    case 2:  // Virtual devices
+    {
+      // Left & right ports
+      myLeftPort->setSelected("left", "left");
+      myRightPort->setSelected("right", "right");
+
+      // Joystick deadzone
+      myDeadzone->setValue(0);
+      myDeadzoneLabel->setValue(3200);
+
+      // Mouse/paddle enabled
+      myMouseEnabled->setState(true);
+
+      // Grab mouse
+      myGrabMouse->setState(true);
+
+      // Paddle speed (digital and mouse)
+      myDPaddleSpeed->setValue(5);
+      myDPaddleLabel->setLabel("5");
+      myMPaddleSpeed->setValue(6);
+      myMPaddleLabel->setLabel("6");
+
+      // AtariVox serial port
+      myAVoxPort->setEditString("");
+
+      // Allow all 4 joystick directions
+      myAllowAll4->setState(false);
+      break;
+    }
+
+    default:
+      break;
+  }
+
+  _dirty = true;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void InputDialog::handleKeyDown(int ascii, int keycode, int modifiers)
 {
@@ -304,6 +392,10 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       close();
       break;
 
+    case kDefaultsCmd:
+      setDefaults();
+      break;
+
     case kLeftChanged:
       myRightPort->setSelected(
         myLeftPort->getSelected() == 1 ? 0 : 1);
@@ -315,11 +407,15 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kDeadzoneChanged:
-      myDeadzoneLabel->setValue(myDeadzone->getValue());
+      myDeadzoneLabel->setValue(3200 + 1000*myDeadzone->getValue());
       break;
 
-    case kPSpeedChanged:
-      myPaddleLabel->setValue(myPaddleSpeed->getValue());
+    case kDPSpeedChanged:
+      myDPaddleLabel->setValue(myDPaddleSpeed->getValue());
+      break;
+
+    case kMPSpeedChanged:
+      myMPaddleLabel->setValue(myMPaddleSpeed->getValue());
       break;
 
     default:
