@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: LauncherDialog.cxx 2001 2010-04-10 21:37:23Z stephena $
+// $Id: LauncherDialog.cxx 2251 2011-06-09 20:50:19Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -35,6 +35,7 @@
 #include "OptionsDialog.hxx"
 #include "GlobalPropsDialog.hxx"
 #include "LauncherFilterDialog.hxx"
+#include "MessageBox.hxx"
 #include "OSystem.hxx"
 #include "Props.hxx"
 #include "PropsSet.hxx"
@@ -61,6 +62,8 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
     myMenu(NULL),
     myGlobalProps(NULL),
     myFilters(NULL),
+    myFirstRunMsg(NULL),
+    myRomDir(NULL),
     mySelectedItem(0)
 {
   const GUI::Font& font = instance().launcherFont();
@@ -191,8 +194,7 @@ LauncherDialog::LauncherDialog(OSystem* osystem, DialogContainer* parent,
   // ROM properties
   myGlobalProps = new GlobalPropsDialog(this, osystem->font());
 
-  // Create dialog whereby the files shown in the ROM listing can be
-  // customized
+  // Create dialog whereby the files shown in the ROM listing can be customized
   myFilters = new LauncherFilterDialog(this, osystem->font());
 
   // Figure out which filters are needed for the ROM listing
@@ -207,16 +209,17 @@ LauncherDialog::~LauncherDialog()
   delete myMenu;
   delete myGlobalProps;
   delete myFilters;
+  delete myRomDir;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string LauncherDialog::selectedRomMD5()
+const string& LauncherDialog::selectedRomMD5()
 {
   string extension;
   int item = myList->getSelected();
   if(item < 0 || myGameList->isDir(item) ||
      !LauncherFilterDialog::isValidRomName(myGameList->name(item), extension))
-    return "";
+    return EmptyString;
 
   // Make sure we have a valid md5 for this ROM
   if(myGameList->md5(item) == "")
@@ -230,12 +233,33 @@ string LauncherDialog::selectedRomMD5()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::loadConfig()
 {
+  const string& romdir = instance().settings().getString("romdir");
+
+  // When romdir hasn't been set, it probably indicates that this is the first
+  // time running Stella; in this case, we should prompt the user
+  if(romdir == "")
+  {
+    if(!myFirstRunMsg)
+    {
+      StringList msg;
+      msg.push_back("This seems to be your first time running Stella.");
+      msg.push_back("Before you can start a game, you need to");
+      msg.push_back("specify where your ROMs are located.");
+      msg.push_back("");
+      msg.push_back("Click 'OK' to select a default ROM directory,");
+      msg.push_back("or 'Cancel' to browse the filesystem manually.");
+      myFirstRunMsg = new MessageBox(this, instance().font(), msg,
+                                     _w, _h, kFirstRunMsgChosenCmd);
+    }
+    myFirstRunMsg->show();
+  }
+
   // Assume that if the list is empty, this is the first time that loadConfig()
-  // has been called (and we should reload the list).
+  // has been called (and we should reload the list)
   if(myList->getList().isEmpty())
   {
     myPrevDirButton->setEnabled(false);
-    myCurrentNode = FilesystemNode(instance().settings().getString("romdir"));
+    myCurrentNode = FilesystemNode(romdir == "" ? "~" : romdir);
     if(!(myCurrentNode.exists() && myCurrentNode.isDirectory()))
       myCurrentNode = FilesystemNode("~");
 
@@ -263,14 +287,13 @@ void LauncherDialog::updateListing(const string& nameToSelect)
   myGameList->clear();
   myDir->setLabel("");
 
-  string romdir = instance().settings().getString("romdir");
   loadDirListing();
 
   // Only hilite the 'up' button if there's a parent directory
   myPrevDirButton->setEnabled(myCurrentNode.hasParent());
 
   // Show current directory
-  myDir->setLabel(myCurrentNode.getRelativePath());
+  myDir->setLabel(myCurrentNode.getPath(false));
 
   // Now fill the list widget with the contents of the GameList
   StringList l;
@@ -410,7 +433,7 @@ void LauncherDialog::setListFilters()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool LauncherDialog::matchPattern(const string& s, const string& pattern)
+bool LauncherDialog::matchPattern(const string& s, const string& pattern) const
 {
   // This method is modelled after strcasestr, which we don't use
   // because it isn't guaranteed to be available everywhere
@@ -503,7 +526,7 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
             if(!myNodeNames.empty())
               dirname = myNodeNames.pop();
           }
-		  else
+          else
           {
             myCurrentNode = FilesystemNode(rom);
             myNodeNames.push(myGameList->name(item));
@@ -515,14 +538,11 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
           if(LauncherFilterDialog::isValidRomName(rom, extension))
           {
             if(instance().createConsole(rom, md5))
-            {
-            #if !defined(GP2X)   // Quick GP2X hack to spare flash-card saves
               instance().settings().setString("lastrom", myList->getSelectedString());
-            #endif
-            }
             else
               instance().frameBuffer().showMessage(
-                  "Error creating console (screen too small)", kMiddleCenter, true);
+                  "Error creating console (check ROM file)",
+                  kMiddleCenter, true);
           }
           else
             instance().frameBuffer().showMessage("Not a valid ROM file",
@@ -551,6 +571,21 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
       instance().eventHandler().quit();
       break;
 
+    case kFirstRunMsgChosenCmd:
+      // Show a file browser, starting from the users' home directory
+      if(!myRomDir)
+        myRomDir = new BrowserDialog(this, instance().font(), _w, _h);
+
+      myRomDir->show("Select ROM directory:", "~",
+                     FilesystemNode::kListDirectoriesOnly, kStartupRomDirChosenCmd);
+      break;
+
+    case kStartupRomDirChosenCmd:
+    {
+      FilesystemNode dir(myRomDir->getResult());
+      instance().settings().setString("romdir", dir.getPath(false));
+      // fall through to the next case
+    }
     case kRomDirChosenCmd:
       myCurrentNode = FilesystemNode(instance().settings().getString("romdir"));
       if(!(myCurrentNode.exists() && myCurrentNode.isDirectory()))
