@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RomAuditDialog.cxx 2125 2010-09-06 00:17:51Z stephena $
+// $Id: RomAuditDialog.cxx 2250 2011-06-09 14:00:30Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -22,6 +22,7 @@
 
 #include "bspf.hxx"
 
+#include "Launcher.hxx"
 #include "LauncherFilterDialog.hxx"
 #include "BrowserDialog.hxx"
 #include "DialogContainer.hxx"
@@ -107,7 +108,12 @@ RomAuditDialog::~RomAuditDialog()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomAuditDialog::loadConfig()
 {
-  myRomPath->setEditString(instance().settings().getString("romdir"));
+  const string& currentdir =
+    instance().launcher().currentNode().getPath(false);
+  const string& path = currentdir == "" ?
+    instance().settings().getString("romdir") : currentdir;
+
+  myRomPath->setEditString(path);
   myResults1->setLabel("");
   myResults2->setLabel("");
 }
@@ -192,6 +198,7 @@ void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
 
     case kConfirmAuditCmd:
       auditRoms();
+      instance().launcher().reload();
       break;
 
     case kChooseAuditDirCmd:
@@ -202,7 +209,7 @@ void RomAuditDialog::handleCommand(CommandSender* sender, int cmd,
     case kAuditDirChosenCmd:
     {
       FilesystemNode dir(myBrowser->getResult());
-      myRomPath->setEditString(dir.getPath());
+      myRomPath->setEditString(dir.getPath(false));
       myResults1->setLabel("");
       myResults2->setLabel("");
       break;
