@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: VideoDialog.cxx 2155 2010-10-18 18:39:57Z stephena $
+// $Id: VideoDialog.cxx 2228 2011-05-06 14:29:39Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -213,12 +213,6 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
   wid.push_back(myUseVSyncCheckbox);
   ypos += lineHeight + 4;
 
-  // Grab mouse (in windowed mode)
-  myGrabmouseCheckbox = new CheckboxWidget(myTab, font, xpos, ypos,
-                                           "Grab mouse");
-  wid.push_back(myGrabmouseCheckbox);
-  ypos += lineHeight + 4;
-
   // Show UI messages onscreen
   myUIMessagesCheckbox = new CheckboxWidget(myTab, font, xpos, ypos,
                                             "Show UI messages");
@@ -344,7 +338,6 @@ VideoDialog::VideoDialog(OSystem* osystem, DialogContainer* parent,
 #endif
 #ifndef WINDOWED_SUPPORT
   myFullscreenCheckbox->clearFlags(WIDGET_ENABLED);
-  myGrabmouseCheckbox->clearFlags(WIDGET_ENABLED);
   myCenterCheckbox->clearFlags(WIDGET_ENABLED);
 #endif
 #if !(defined(BSPF_WIN32) || (defined(BSPF_UNIX) && defined(HAVE_X11)))
@@ -425,9 +418,6 @@ void VideoDialog::loadConfig()
   myUseVSyncCheckbox->setState(instance().settings().getBool("gl_vsync"));
   myUseVSyncCheckbox->setEnabled(gl);
 
-  // Grab mouse
-  myGrabmouseCheckbox->setState(instance().settings().getBool("grabmouse"));
-
   // Show UI messages
   myUIMessagesCheckbox->setState(instance().settings().getBool("uimessages"));
 
@@ -522,10 +512,6 @@ void VideoDialog::saveConfig()
   // Use sync to vertical blank (GL mode only)
   instance().settings().setBool("gl_vsync", myUseVSyncCheckbox->getState());
 
-  // Grab mouse
-  instance().settings().setBool("grabmouse", myGrabmouseCheckbox->getState());
-  instance().frameBuffer().setCursorState();
-
   // Show UI messages
   instance().settings().setBool("uimessages", myUIMessagesCheckbox->getState());
 
@@ -572,7 +558,6 @@ void VideoDialog::setDefaults()
   myColorLossCheckbox->setState(false);
   myGLStretchCheckbox->setState(false);
   myUseVSyncCheckbox->setState(true);
-  myGrabmouseCheckbox->setState(false);
   myUIMessagesCheckbox->setState(true);
   myCenterCheckbox->setState(false);
   myFastSCBiosCheckbox->setState(false);
