@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FSNodePOSIX.cxx 2125 2010-09-06 00:17:51Z stephena $
+// $Id: FSNodePOSIX.cxx 2252 2011-06-11 15:58:42Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -58,17 +58,16 @@ class POSIXFilesystemNode : public AbstractFilesystemNode
      */
     POSIXFilesystemNode(const string& path, bool verify);
 
-    virtual bool exists() const { return access(_path.c_str(), F_OK) == 0; }
-    virtual string getDisplayName() const { return _displayName; }
-    virtual string getName() const   { return _displayName; }
-    virtual string getPath() const   { return _path; }
-    virtual string getRelativePath() const;
-    virtual bool isDirectory() const { return _isDirectory; }
-    virtual bool isReadable() const  { return access(_path.c_str(), R_OK) == 0; }
-    virtual bool isWritable() const  { return access(_path.c_str(), W_OK) == 0; }
+    bool exists() const { return access(_path.c_str(), F_OK) == 0; }
+    string getDisplayName() const { return _displayName; }
+    string getName() const   { return _displayName; }
+    string getPath(bool fqn) const;
+    bool isDirectory() const { return _isDirectory; }
+    bool isReadable() const  { return access(_path.c_str(), R_OK) == 0; }
+    bool isWritable() const  { return access(_path.c_str(), W_OK) == 0; }
 
-    virtual bool getChildren(AbstractFSList& list, ListMode mode, bool hidden) const;
-    virtual AbstractFilesystemNode* getParent() const;
+    bool getChildren(AbstractFSList& list, ListMode mode, bool hidden) const;
+    AbstractFilesystemNode* getParent() const;
 
   protected:
     string _displayName;
@@ -130,27 +129,18 @@ POSIXFilesystemNode::POSIXFilesystemNode()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 POSIXFilesystemNode::POSIXFilesystemNode(const string& p, bool verify)
 {
-  // Expand "~/" to the value of the HOME env variable
-  if ( p.length() >= 2 && p[0] == '~' && p[1] == '/')
+  // Expand '~/' and './' to the value of the HOME env variable
+  if ( p.length() >= 2 && (p[0] == '~' || p[0] == '.') && p[1] == '/')
   {
     const char *home = getenv("HOME");
+#ifdef MAXPATHLEN
     if (home != NULL && strlen(home) < MAXPATHLEN)
+#else // No MAXPATHLEN, as happens on Hurd
+    if (home != NULL)
+#endif
     {
       _path = home;
-      // Skip over the tilde.  We know that p contains at least
-      // two chars, so this is safe:
-      _path += p.c_str() + 1;
-    }
-  }
-  // Expand "./" to the current directory
-  else if ( p.length() >= 2 && p[0] == '.' && p[1] == '/')
-  {
-    char buf[MAXPATHLEN];
-    char* ret = getcwd(buf, MAXPATHLEN);
-    if (ret == buf)
-    {
-      _path = buf;
-      // Skip over the tilda.  We know that p contains at least
+      // Skip over the tilde/dot.  We know that p contains at least
       // two chars, so this is safe:
       _path += p.c_str() + 1;
     }
@@ -171,21 +161,17 @@ POSIXFilesystemNode::POSIXFilesystemNode(const string& p, bool verify)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string POSIXFilesystemNode::getRelativePath() const
+string POSIXFilesystemNode::getPath(bool fqn) const
 {
   // If the path starts with the home directory, replace it with '~'
   const char* home = getenv("HOME");
-  if(home != NULL)
+  if(!fqn && home != NULL && BSPF_startsWithIgnoreCase(_path, home))
   {
-    int len = strlen(home);
-    if(strncmp(_path.c_str(), home, len) == 0)
-    {
-      string path = "~";
-      const char* offset = _path.c_str() + len;
-      if(*offset != '/') path += "/";
-      path += offset;
-      return path;
-    }
+    string path = "~";
+    const char* offset = _path.c_str() + strlen(home);
+    if(*offset != '/') path += "/";
+    path += offset;
+    return path;
   }
   return _path;
 }
@@ -289,12 +275,6 @@ AbstractFilesystemNode* AbstractFilesystemNode::makeRootFileNode()
   return new POSIXFilesystemNode();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-AbstractFilesystemNode* AbstractFilesystemNode::makeCurrentDirectoryFileNode()
-{
-  return new POSIXFilesystemNode("./", true);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AbstractFilesystemNode* AbstractFilesystemNode::makeHomeDirectoryFileNode()
 {
@@ -319,3 +299,29 @@ bool AbstractFilesystemNode::renameFile(const string& oldfile,
 {
   return rename(oldfile.c_str(), newfile.c_str()) == 0;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string AbstractFilesystemNode::getAbsolutePath(const string& p,
+                                               const string& startpath,
+                                               const string& ext)
+{
+  // Does p start with the root directory or the given startpath?
+  // If not, it isn't an absolute path
+  string path = FilesystemNode(p).getPath(false);
+  if(!BSPF_startsWithIgnoreCase(p, startpath+"/") &&
+     !BSPF_startsWithIgnoreCase(p, "/"))
+    path = startpath + "/" + p;
+
+  // Does the path have a valid extension?
+  // If not, we add the given one
+  string::size_type idx = path.find_last_of('.');
+  if(idx != string::npos)
+  {
+    if(!BSPF_equalsIgnoreCase(path.c_str() + idx + 1, ext))
+      path = path.replace(idx+1, ext.length(), ext);
+  }
+  else
+    path += "." + ext;
+
+  return path;
+}
