@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FSNodeWin32.cxx 2125 2010-09-06 00:17:51Z stephena $
+// $Id: FSNodeWin32.cxx 2253 2011-06-11 15:59:19Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -91,17 +91,16 @@ class WindowsFilesystemNode : public AbstractFilesystemNode
      */
     WindowsFilesystemNode(const string& path);
 
-    virtual bool exists() const { return _access(_path.c_str(), F_OK) == 0; }
-    virtual string getDisplayName() const { return _displayName; }
-    virtual string getName() const   { return _displayName; }
-    virtual string getPath() const   { return _path; }
-    virtual string getRelativePath() const;
-    virtual bool isDirectory() const { return _isDirectory; }
-    virtual bool isReadable() const  { return _access(_path.c_str(), R_OK) == 0; }
-    virtual bool isWritable() const  { return _access(_path.c_str(), W_OK) == 0; }
+    bool exists() const { return _access(_path.c_str(), F_OK) == 0; }
+    string getDisplayName() const { return _displayName; }
+    string getName() const   { return _displayName; }
+    string getPath(bool fqn) const;
+    bool isDirectory() const { return _isDirectory; }
+    bool isReadable() const  { return _access(_path.c_str(), R_OK) == 0; }
+    bool isWritable() const  { return _access(_path.c_str(), W_OK) == 0; }
 
-    virtual bool getChildren(AbstractFSList& list, ListMode mode, bool hidden) const;
-    virtual AbstractFilesystemNode* getParent() const;
+    bool getChildren(AbstractFSList& list, ListMode mode, bool hidden) const;
+    AbstractFilesystemNode* getParent() const;
 
   protected:
     string _displayName;
@@ -115,17 +114,17 @@ class WindowsFilesystemNode : public AbstractFilesystemNode
      * Adds a single WindowsFilesystemNode to a given list.
      * This method is used by getChildren() to populate the directory entries list.
      *
-     * @param list List to put the file entry node in.
-     * @param mode Mode to use while adding the file entry to the list.
-     * @param base String with the directory being listed.
-     * @param find_data Describes a file that the FindFirstFile, FindFirstFileEx, or FindNextFile functions find.
+     * @param list       List to put the file entry node in.
+     * @param mode       Mode to use while adding the file entry to the list.
+     * @param base       String with the directory being listed.
+     * @param find_data  Describes a file that the FindFirstFile, FindFirstFileEx, or FindNextFile functions find.
      */
     static void addFile(AbstractFSList& list, ListMode mode, const char* base, WIN32_FIND_DATA* find_data);
 
     /**
      * Converts a Unicode string to Ascii format.
      *
-     * @param str String to convert from Unicode to Ascii.
+     * @param str  String to convert from Unicode to Ascii.
      * @return str in Ascii format.
      */
     static char* toAscii(TCHAR *str);
@@ -133,7 +132,7 @@ class WindowsFilesystemNode : public AbstractFilesystemNode
     /**
      * Converts an Ascii string to Unicode format.
      *
-     * @param str String to convert from Ascii to Unicode.
+     * @param str  String to convert from Ascii to Unicode.
      * @return str in Unicode format.
      */
     static const TCHAR* toUnicode(const char* str);
@@ -239,21 +238,11 @@ WindowsFilesystemNode::WindowsFilesystemNode()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 WindowsFilesystemNode::WindowsFilesystemNode(const string& p)
 {
-  // Expand "~\" to the 'My Documents' directory
-  if ( p.length() >= 2 && p[0] == '~' && p[1] == '\\')
+  // Expand '~\' and '.\' to the users 'home' directory
+  if ( p.length() >= 2 && (p[0] == '~' || p[0] == '.') && p[1] == '\\')
   {
     _path = myHomeFinder.getHomePath();
-    // Skip over the tilda.  We know that p contains at least
-    // two chars, so this is safe:
-    _path += p.c_str() + 1;
-  }
-  // Expand ".\" to the current directory
-  else if ( p.length() >= 2 && p[0] == '.' && p[1] == '\\')
-  {
-    char path[MAX_PATH];
-    GetCurrentDirectory(MAX_PATH, path);
-    _path = path;
-    // Skip over the tilda.  We know that p contains at least
+    // Skip over the tilde/dot.  We know that p contains at least
     // two chars, so this is safe:
     _path += p.c_str() + 1;
   }
@@ -286,22 +275,17 @@ WindowsFilesystemNode::WindowsFilesystemNode(const string& p)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string WindowsFilesystemNode::getRelativePath() const
+string WindowsFilesystemNode::getPath(bool fqn) const
 {
   // If the path starts with the home directory, replace it with '~'
   const string& home = myHomeFinder.getHomePath();
-  if(home != "")
+  if(!fqn && home != "" && BSPF_startsWithIgnoreCase(_path, home))
   {
-    // Windows file system not case sensitive
-    int len = home.length();
-    if(BSPF_strncasecmp(home.c_str(), _path.substr(0, len).c_str(), len) == 0)
-    {
-      string path = "~";
-      const char* offset = _path.c_str() + len;
-      if(*offset != '\\') path += '\\';
-      path += offset;
-      return path;
-    }
+    string path = "~";
+    const char* offset = _path.c_str() + home.length();
+    if(*offset != '\\') path += '\\';
+    path += offset;
+    return path;
   }
   return _path;
 }
@@ -393,12 +377,6 @@ AbstractFilesystemNode* AbstractFilesystemNode::makeRootFileNode()
   return new WindowsFilesystemNode();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-AbstractFilesystemNode* AbstractFilesystemNode::makeCurrentDirectoryFileNode()
-{
-  return new WindowsFilesystemNode(".\\");
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 AbstractFilesystemNode* AbstractFilesystemNode::makeHomeDirectoryFileNode()
 {
@@ -423,3 +401,29 @@ bool AbstractFilesystemNode::renameFile(const string& oldfile,
 {
   return MoveFile(oldfile.c_str(), newfile.c_str()) != 0;
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string AbstractFilesystemNode::getAbsolutePath(const string& p,
+                                               const string& startpath,
+                                               const string& ext)
+{
+  // Does p start with a drive letter or the given startpath?
+  // If not, it isn't an absolute path
+  string path = FilesystemNode(p).getPath(false);
+  bool startsWithDrive = path.length() >= 2 && path[1] == ':';
+  if(!BSPF_startsWithIgnoreCase(p, startpath+"\\") && !startsWithDrive)
+    path = startpath + "\\" + p;
+
+  // Does the path have a valid extension?
+  // If not, we add the given one
+  string::size_type idx = path.find_last_of('.');
+  if(idx != string::npos)
+  {
+    if(!BSPF_equalsIgnoreCase(path.c_str() + idx + 1, ext))
+      path = path.replace(idx+1, ext.length(), ext);
+  }
+  else
+    path += "." + ext;
+
+  return path;
+}
