@@ -8,16 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2010 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: YaccParser.cxx 2001 2010-04-10 21:37:23Z stephena $
-//
-//   Based on code from ScummVM - Scumm Interpreter
-//   Copyright (C) 2002-2004 The ScummVM project
+// $Id: YaccParser.cxx 2199 2011-01-01 16:04:32Z stephena $
 //============================================================================
 
 //#include "YaccParser.hxx"
@@ -81,23 +78,27 @@ int parse(const char *in)
 }
 
 /* hand-rolled lexer. Hopefully faster than flex... */
-inline bool is_base_prefix(char x) { return ( (x=='\\' || x=='$' || x=='#') ); }
-
-inline bool is_identifier(char x) {
-  return ( (x>='0' && x<='9') || 
-          (x>='a' && x<='z') ||
-          (x>='A' && x<='Z') ||
-           x=='.' || x=='_'  );
+inline bool is_base_prefix(char x)
+{
+  return ( (x=='\\' || x=='$' || x=='#') );
 }
 
+inline bool is_identifier(char x)
+{
+  return ( (x>='0' && x<='9') || 
+           (x>='a' && x<='z') ||
+           (x>='A' && x<='Z') ||
+            x=='.' || x=='_'  );
+}
 
-inline bool is_operator(char x) {
+inline bool is_operator(char x)
+{
   return ( (x=='+' || x=='-' || x=='*' ||
-             x=='/' || x=='<' || x=='>' ||
-             x=='|' || x=='&' || x=='^' ||
-             x=='!' || x=='~' || x=='(' ||
-             x==')' || x=='=' || x=='%' ||
-         x=='[' || x==']' ) );
+            x=='/' || x=='<' || x=='>' ||
+            x=='|' || x=='&' || x=='^' ||
+            x=='!' || x=='~' || x=='(' ||
+            x==')' || x=='=' || x=='%' ||
+            x=='[' || x==']' ) );
 }
 
 // const_to_int converts a string into a number, in either the
@@ -171,29 +172,29 @@ int const_to_int(char *c) {
 // special methods that get e.g. CPU registers
 CPUDEBUG_INT_METHOD getCpuSpecial(char *c)
 {
-  if(strcmp(c, "a") == 0)
+  if(BSPF_equalsIgnoreCase(c, "a"))
     return &CpuDebug::a;
-  else if(strcmp(c, "x") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "x"))
     return &CpuDebug::x;
-  else if(strcmp(c, "y") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "y"))
     return &CpuDebug::y;
-  else if(strcmp(c, "pc") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "pc"))
     return &CpuDebug::pc;
-  else if(strcmp(c, "sp") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "sp"))
     return &CpuDebug::sp;
-  else if(strcmp(c, "c") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "c"))
     return &CpuDebug::c;
-  else if(strcmp(c, "z") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "z"))
     return &CpuDebug::z;
-  else if(strcmp(c, "n") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "n"))
     return &CpuDebug::n;
-  else if(strcmp(c, "v") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "v"))
     return &CpuDebug::v;
-  else if(strcmp(c, "d") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "d"))
     return &CpuDebug::d;
-  else if(strcmp(c, "i") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "i"))
     return &CpuDebug::i;
-  else if(strcmp(c, "b") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "b"))
     return &CpuDebug::b;
   else
     return 0;
@@ -202,9 +203,9 @@ CPUDEBUG_INT_METHOD getCpuSpecial(char *c)
 // special methods that get Cart RAM/ROM internal state
 CARTDEBUG_INT_METHOD getCartSpecial(char *c)
 {
-  if(strcmp(c, "_bank") == 0)
+  if(BSPF_equalsIgnoreCase(c, "_bank"))
     return &CartDebug::getBank;
-  else if(strcmp(c, "_rwport") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "_rwport"))
     return &CartDebug::readFromWritePort;
   else
     return 0;
@@ -213,15 +214,15 @@ CARTDEBUG_INT_METHOD getCartSpecial(char *c)
 // special methods that get TIA internal state
 TIADEBUG_INT_METHOD getTiaSpecial(char *c)
 {
-  if(strcmp(c, "_scan") == 0)
+  if(BSPF_equalsIgnoreCase(c, "_scan"))
     return &TIADebug::scanlines;
-  else if(strcmp(c, "_fcount") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "_fcount"))
     return &TIADebug::frameCount;
-  else if(strcmp(c, "_cclocks") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "_cclocks"))
     return &TIADebug::clocksThisLine;
-  else if(strcmp(c, "_vsync") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "_vsync"))
     return &TIADebug::vsyncAsInt;
-  else if(strcmp(c, "_vblank") == 0)
+  else if(BSPF_equalsIgnoreCase(c, "_vblank"))
     return &TIADebug::vblankAsInt;
   else
     return 0;
